// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/driver/ioctl.c $
// $Rev: 53435 $
// $Date: 2023-07-24 10:01:26 -0500 (Mon, 24 Jul 2023) $

// 16HSDI4AO4: Device Driver: source file

#include "main.h"



// macros *********************************************************************

#define	AUTOCAL_START			D28			// BCR
#define	AUTOCAL_PASS			D29			// BCR
#define	INIT_START				D31			// BCR

#define	IRQ_AUTOCAL_DONE		0x0001		// PSR



//*****************************************************************************
static void _psr_mod(dev_data_t* dev, u32 value, u32 mask)
{
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_psr_32, value, mask);
}



//*****************************************************************************
static int _query(dev_data_t* dev, s32* arg)
{
	switch (arg[0])
	{
		default:								arg[0]	= HSDI4AO4_IOCTL_QUERY_ERROR;	break;
		case HSDI4AO4_QUERY_AUTOCAL_MS:			arg[0]	= dev->cache.autocal_ms;		break;
		case HSDI4AO4_QUERY_CHANNEL_AI_MAX:		arg[0]	= dev->cache.ai_chans_max;		break;
		case HSDI4AO4_QUERY_CHANNEL_AI_QTY:		arg[0]	= dev->cache.ai_chan_qty;		break;
		case HSDI4AO4_QUERY_CHANNEL_AO_MAX:		arg[0]	= dev->cache.ao_chans_max;		break;
		case HSDI4AO4_QUERY_CHANNEL_AO_QTY:		arg[0]	= dev->cache.ao_chan_qty;		break;
		case HSDI4AO4_QUERY_COUNT:				arg[0]	= HSDI4AO4_IOCTL_QUERY_LAST;	break;
		case HSDI4AO4_QUERY_DEVICE_TYPE:		arg[0]	= dev->board_type;				break;
		case HSDI4AO4_QUERY_FIFO_SIZE_RX:		arg[0]	= dev->cache.fifo_size_rx;		break;
		case HSDI4AO4_QUERY_FIFO_SIZE_TX:		arg[0]	= dev->cache.fifo_size_tx;		break;
		case HSDI4AO4_QUERY_FSAMP_MAX:			arg[0]	= dev->cache.fsamp_max;			break;
		case HSDI4AO4_QUERY_FSAMP_MIN:			arg[0]	= dev->cache.fsamp_min;			break;
		case HSDI4AO4_QUERY_GEN_A_MAX:			arg[0]	= dev->cache.gen_a_max;			break;
		case HSDI4AO4_QUERY_GEN_A_MIN:			arg[0]	= dev->cache.gen_a_min;			break;
		case HSDI4AO4_QUERY_GEN_A_NDIV_MAX:		arg[0]	= dev->cache.gen_a_ndiv_max;	break;
		case HSDI4AO4_QUERY_GEN_A_NDIV_MIN:		arg[0]	= dev->cache.gen_a_ndiv_min;	break;
		case HSDI4AO4_QUERY_GEN_A_NREF_MAX:		arg[0]	= dev->cache.gen_a_nref_max;	break;
		case HSDI4AO4_QUERY_GEN_A_NREF_MIN:		arg[0]	= dev->cache.gen_a_nref_min;	break;
		case HSDI4AO4_QUERY_GEN_A_NVCO_MAX:		arg[0]	= dev->cache.gen_a_nvco_max;	break;
		case HSDI4AO4_QUERY_GEN_A_NVCO_MIN:		arg[0]	= dev->cache.gen_a_nvco_min;	break;
		case HSDI4AO4_QUERY_GEN_B_NDIV_MAX:		arg[0]	= dev->cache.gen_b_ndiv_max;	break;
		case HSDI4AO4_QUERY_GEN_B_NDIV_MIN:		arg[0]	= dev->cache.gen_b_ndiv_min;	break;
		case HSDI4AO4_QUERY_GEN_C_NDIV_MAX:		arg[0]	= dev->cache.gen_c_ndiv_max;	break;
		case HSDI4AO4_QUERY_GEN_C_NDIV_MIN:		arg[0]	= dev->cache.gen_c_ndiv_min;	break;
		case HSDI4AO4_QUERY_INIT_MS:			arg[0]	= dev->cache.initialize_ms;		break;
		case HSDI4AO4_QUERY_MASTER_CLOCK_DEF:	arg[0]	= dev->cache.master_clock_def;	break;
		case HSDI4AO4_QUERY_MCAR:				arg[0]	= dev->cache.mcar;				break;
	}

	return(0);
}



//*****************************************************************************
int initialize_ioctl(dev_data_t* dev, void* arg)
{
	int			i;
	long		ms			= dev->cache.initialize_ms + 5000;
	u32			reg;
	int			ret			= 0;
	int			tmp;
	VADDR_T		va			= dev->vaddr.gsc_bcr_32;

	// This device doesn't support an INITIALIZE DONE interrupt.

	// Initiate initialization.
	os_reg_mem_tx_u32(dev, va, INIT_START);

	// Manually wait for completion as this device dowsn't support an
	// Initialize Done interrupt.
	tmp	= gsc_poll_u32(dev, ms, va, INIT_START, 0);

	if (tmp)
	{
		ret	= ret ? ret : tmp;
		printf(	"%s: INITIALIZATION DID NOT COMPLETE WITHIN %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually check for completion as the IRQ wait may have ended early.
	reg	= os_reg_mem_rx_u32(dev, va);

	if (reg & INIT_START)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: INITILIZE STILL ACTIVE AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Initialize the software settings.
	dev->master_clock	= dev->cache.master_clock_def;

	for (i = 0; i < DEV_IO_STREAM_QTY; i++)
	{
		if (dev->io.io_streams[i])
		{
			if (dev->io.io_streams[i]->dev_io_sw_init)
			{
				(dev->io.io_streams[i]->dev_io_sw_init)(dev, dev->io.io_streams[i]);
			}
		}
	}

	return(ret);
}



//*****************************************************************************
static int _autocal_start(dev_data_t* dev, void* arg)
{
	// Initiate autocalibration.
	os_reg_mem_mx_u32(NULL, dev->vaddr.gsc_bcr_32, AUTOCAL_START, AUTOCAL_START);
	return(0);
}



//*****************************************************************************
static int _autocal(dev_data_t* dev, void* arg)
{
	u32			mask;
	long		ms		= dev->cache.autocal_ms + 5000;
	u32			reg;
	int			ret;
	os_sem_t	sem;
	int			tmp;
	VADDR_T		va		= dev->vaddr.gsc_bcr_32;
	u32			value;
	gsc_wait_t	wt;

	// Safely select the Autocal Done interrupt.
	mask	= IRQ_AUTOCAL_DONE;
	value	= IRQ_AUTOCAL_DONE;
	_psr_mod(dev, value, mask);

	// Wait for the local interrupt.
	os_sem_create(&sem);	// dummy, required for wait operations.
	memset(&wt, 0, sizeof(wt));
	wt.flags		= GSC_WAIT_FLAG_INTERNAL;
	wt.gsc			= HSDI4AO4_WAIT_GSC_AUTOCAL_DONE;
	wt.timeout_ms	= ms;
	ret				= gsc_wait_event(dev, &wt, _autocal_start, NULL, &sem);
	os_sem_destroy(&sem);

	if (wt.flags & GSC_WAIT_FLAG_TIMEOUT)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: AUTOCALIBRATE DONE IRQ TIMED OUT AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually wait for completion in case something terminates our wait early.
	tmp	= gsc_poll_u32(dev, ms, va, AUTOCAL_START, 0);

	if (tmp)
	{
		ms	*= 2;
		ret	= ret ? ret : tmp;
		printf(	"%s: AUTOCALIBRATION DID NOT COMPLETE WITHIN %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually check for completion as the IRQ wait may have ended early.
	reg	= os_reg_mem_rx_u32(dev, va);

	if (reg & AUTOCAL_START)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: AUTOCALIBRATION STILL ACTIVE AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Final results.
	reg	= os_reg_mem_rx_u32(dev, va);

	if ((reg & AUTOCAL_PASS) == 0)
	{
		ret	= ret ? ret : -EIO;
		printf(	"%s: AUTOCALIBRATION FAILED (%ld ms).\n",
				dev->model,
				ms);
	}

	return(ret);
}



//*****************************************************************************
static int _autocal_status(dev_data_t* dev, s32* arg)
{
	u32	reg;

	reg	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if (reg & AUTOCAL_START)
		arg[0]	= HSDI4AO4_AUTOCAL_STATUS_ACTIVE;
	else if (reg & AUTOCAL_PASS)
		arg[0]	= HSDI4AO4_AUTOCAL_STATUS_PASS;
	else
		arg[0]	= HSDI4AO4_AUTOCAL_STATUS_FAIL;

	return(0);
}



//*****************************************************************************
static int _rx_io_mode(dev_data_t* dev, s32* arg)
{
	static const s32	list[]	=
	{
		GSC_IO_MODE_PIO,
		GSC_IO_MODE_BMDMA,
		GSC_IO_MODE_DMDMA,
		-1
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, list, &dev->io.rx.io_mode);
	return(ret);
}



//*****************************************************************************
static int _rx_io_overflow(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_IO_OVERFLOW_IGNORE,
		HSDI4AO4_IO_OVERFLOW_CHECK,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.rx.overflow_check);
	return(ret);
}



//*****************************************************************************
static int _rx_io_timeout(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_var(
			arg,
			HSDI4AO4_IO_TIMEOUT_MIN,
			HSDI4AO4_IO_TIMEOUT_INFINITE,
			&dev->io.rx.timeout_s);
	return(ret);
}



//*****************************************************************************
static int _rx_io_underflow(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_IO_UNDERFLOW_IGNORE,
		HSDI4AO4_IO_UNDERFLOW_CHECK,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.rx.underflow_check);
	return(ret);
}



//*****************************************************************************
static int _tx_io_mode(dev_data_t* dev, s32* arg)
{
	static const s32	list[]	=
	{
		GSC_IO_MODE_PIO,
		GSC_IO_MODE_BMDMA,
		GSC_IO_MODE_DMDMA,
		-1
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, list, &dev->io.tx.io_mode);
	return(ret);
}



//*****************************************************************************
static int _tx_io_overflow(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_IO_OVERFLOW_IGNORE,
		HSDI4AO4_IO_OVERFLOW_CHECK,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.tx.overflow_check);
	return(ret);
}



//*****************************************************************************
static int _tx_io_timeout(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_var(
			arg,
			HSDI4AO4_IO_TIMEOUT_MIN,
			HSDI4AO4_IO_TIMEOUT_INFINITE,
			&dev->io.tx.timeout_s);
	return(ret);
}



//*****************************************************************************
static int _ai_buf_clear(dev_data_t* dev, void* arg)
{
	#define	CLEAR	D13
	#define	OVER	D15
	#define	UNDER	D23

	int		i;
	u32		reg;
	int		ret	= 0;
	VADDR_T	va	= dev->vaddr.gsc_bcr_32;

	// Clear the buffer.
	os_reg_mem_mx_u32(dev, va, CLEAR, CLEAR);

	// Wait for the bit to clear.

	for (i = 0;; i++)
	{
		reg	= os_reg_mem_rx_u32(dev, va);

		if ((reg & CLEAR) == 0)
			break;

		if (i >= 250)
		{
			ret	= -EINVAL;
			printf(	"%s: The analog input buffer took too long to clear.\n",
					DEV_NAME);
			break;
		}
	}

	// Clear the Overflow status bit.
	os_reg_mem_mx_u32(dev, va, 0, OVER);

	// Clear the Underflow status bit.
	os_reg_mem_mx_u32(dev, va, 0, UNDER);

	return(ret);
}



//*****************************************************************************
static int _ai_buf_level(dev_data_t* dev, s32* arg)
{
	arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_ibsr_32);
	arg[0]	&= 0x7FFFF;
	return(0);
}



//*****************************************************************************
static int _ai_buf_overflow(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_BUF_OVERFLOW_NO,
		HSDI4AO4_BUF_OVERFLOW_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 15, 15);

	if ((ret == 0) && (arg[0] == -1))
	{
		arg[0]	= -1;
		ret		= _ai_buf_overflow(dev, arg);
	}

	return(ret);
}



//*****************************************************************************
static int _ai_buf_thr_lvl(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0x7FFFF, dev->vaddr.gsc_ibtr_32, 18, 0);
	return(ret);
}



//*****************************************************************************
static int _ai_buf_thr_sts(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_BUF_THR_STS_CLEAR,
		HSDI4AO4_BUF_THR_STS_SET,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_ibtr_32, 19, 19);
	return(ret);
}



//*****************************************************************************
static int _ai_buf_underflow(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_BUF_UNDERFLOW_NO,
		HSDI4AO4_BUF_UNDERFLOW_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 23, 23);

	if ((ret == 0) && (arg[0] == -1))
	{
		arg[0]	= -1;
		ret		= _ai_buf_underflow(dev, arg);
	}

	return(ret);
}



//*****************************************************************************
static int _ai_burst_enable(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_BURST_ENABLE_NO,
		HSDI4AO4_BURST_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 9, 9);
	return(ret);
}



//*****************************************************************************
static int _ai_burst_size(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFFFF, dev->vaddr.gsc_icr_32, 23, 0);
	return(ret);
}



//*****************************************************************************
static int _ai_burst_status(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_AI_BURST_STATUS_BUSY,
		HSDI4AO4_AI_BURST_STATUS_READY,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 10, 10);
	return(ret);
}



//*****************************************************************************
static int _ai_chan_sel(dev_data_t* dev, void* arg)
{
	int	ret;

	if (dev->cache.ai_chan_qty == 4)
		ret	= gsc_s32_range_reg(dev, arg, 0, 0xF, dev->vaddr.gsc_icr_32, 27, 24);
	else
		ret	= gsc_s32_range_reg(dev, arg, 0, 0x3, dev->vaddr.gsc_icr_32, 25, 24);

	return(ret);
}



//*****************************************************************************
static int _ai_clock_mode(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_CLOCK_MODE_IN,
		HSDI4AO4_CLOCK_MODE_OUT,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 24, 24);
	return(ret);
}



//*****************************************************************************
static int _ai_enable(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_AIO_ENABLE_NO,
		HSDI4AO4_AIO_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 12, 12);
	return(ret);
}



//*****************************************************************************
static int _ai_mode(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_AI_MODE_AO_3,
		HSDI4AO4_AI_MODE_AO_2,
		HSDI4AO4_AI_MODE_AO_1,
		HSDI4AO4_AI_MODE_AO_0,
		HSDI4AO4_AI_MODE_DIFF,
		HSDI4AO4_AI_MODE_SINGLE,
		HSDI4AO4_AI_MODE_ZERO,
		HSDI4AO4_AI_MODE_VREF,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.ao_chan_qty == 4)
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 2, 0);
	else
		ret	= gsc_s32_list_reg(dev, arg, options + 2, dev->vaddr.gsc_bcr_32, 2, 0);

	return(ret);
}



//*****************************************************************************
static int _ai_range(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_RANGE_2_5V,
		HSDI4AO4_RANGE_5V,
		HSDI4AO4_RANGE_10V,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 5, 4);
	return(ret);
}



//*****************************************************************************
static int _ai_sw_clock(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x1L << SHIFT)
	#define	SHIFT	8

	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, MASK, MASK);
	return(0);
}



//*****************************************************************************
static int _ai_sw_trigger(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x1L << SHIFT)
	#define	SHIFT	11

	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, MASK, MASK);
	return(0);
}



//*****************************************************************************
static int _ao_buf_clear(dev_data_t* dev, void* arg)
{
	#undef	CLEAR
	#define	CLEAR	D11
	#define	DATA	D16
	#define	FRAME	D17

	int		i;
	u32		reg;
	int		ret	= 0;
	VADDR_T	va	= dev->vaddr.gsc_boor_32;

	// Clear the buffer.
	os_reg_mem_mx_u32(dev, va, CLEAR, CLEAR);

	// Wait for the bit to clear.

	for (i = 0;; i++)
	{
		reg	= os_reg_mem_rx_u32(dev, va);

		if ((reg & CLEAR) == 0)
			break;

		if (i >= 250)
		{
			ret	= -EINVAL;
			printf(	"%s: The analog output buffer took too long to clear.\n",
					DEV_NAME);
			break;
		}
	}

	// Clear the Data Overflow status bit.
	os_reg_mem_mx_u32(dev, va, 0, DATA);

	// Clear the Frame Overflow status bit.
	os_reg_mem_mx_u32(dev, va, 0, FRAME);

	return(ret);
}



//*****************************************************************************
static int _ao_buf_load_req(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x1L << SHIFT)
	#define	SHIFT	9

	int	ret	= 0;

	// Make the request.
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_boor_32, MASK, MASK);

	// Wait for the request to be granted.

	if (dev->io.tx.timeout_s)
	{
		ret	= gsc_poll_u32(dev, dev->io.tx.timeout_s * 1000, dev->vaddr.gsc_boor_32, 0x400, 0x400);
	}

	return(ret);
}



//*****************************************************************************
static int _ao_buf_load_sts(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_BUF_LOAD_STS_NOT_READY,
		HSDI4AO4_BUF_LOAD_STS_READY,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 10, 10);
	return(ret);
}



//*****************************************************************************
static int _ao_buf_mode(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_BUF_MODE_OPEN,
		HSDI4AO4_BUF_MODE_CIRC,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 8, 8);
	return(ret);
}



//*****************************************************************************
static int _ao_buf_level(dev_data_t* dev, s32* arg)
{
	arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_obsr_32);
	arg[0]	&= 0x7FFFF;
	return(0);
}



//*****************************************************************************
static int _ao_buf_over_data(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_BUF_OVERFLOW_NO,
		HSDI4AO4_BUF_OVERFLOW_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 16, 16);

	if ((ret == 0) && (arg[0] == -1))
	{
		arg[0]	= -1;
		ret		= _ao_buf_over_data(dev, arg);
	}

	return(ret);
}



//*****************************************************************************
static int _ao_buf_over_fram(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_BUF_OVERFLOW_NO,
		HSDI4AO4_BUF_OVERFLOW_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 17, 17);

	if ((ret == 0) && (arg[0] == -1))
	{
		arg[0]	= -1;
		ret		= _ao_buf_over_fram(dev, arg);
	}

	return(ret);
}



//*****************************************************************************
static int _ao_buf_thr_lvl(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0x7FFFF, dev->vaddr.gsc_obtr_32, 18, 0);
	return(ret);
}



//*****************************************************************************
static int _ao_buf_thr_sts(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_BUF_THR_STS_CLEAR,
		HSDI4AO4_BUF_THR_STS_SET,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_obtr_32, 19, 19);
	return(ret);
}



//*****************************************************************************
static int _ao_burst_enable(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_BURST_ENABLE_NO,
		HSDI4AO4_BURST_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 19, 19);
	return(ret);
}



//*****************************************************************************
static int _ao_burst_status(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_AO_BURST_STATUS_BUSY,
		HSDI4AO4_AO_BURST_STATUS_READY,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 18, 18);
	return(ret);
}



//*****************************************************************************
static int _ao_chan_sel(dev_data_t* dev, void* arg)
{
	int	ret;

	if (dev->cache.ao_chan_qty == 4)
		ret	= gsc_s32_range_reg(dev, arg, 0, 0xF, dev->vaddr.gsc_boor_32, 3, 0);
	else
		ret	= -EINVAL;

	return(ret);
}



//*****************************************************************************
static int _ao_clock_status(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_CLOCK_STATUS_NOT_READY,
		HSDI4AO4_CLOCK_STATUS_READY,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 6, 6);
	return(ret);
}



//*****************************************************************************
static int _ao_clock_mode(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_CLOCK_MODE_IN,
		HSDI4AO4_CLOCK_MODE_OUT,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 4, 4);
	return(ret);
}



//*****************************************************************************
static int _ao_enable(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_AIO_ENABLE_NO,
		HSDI4AO4_AIO_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 5, 5);
	return(ret);
}



//*****************************************************************************
static int _ao_mode(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_AO_MODE_REG,
		HSDI4AO4_AO_MODE_FIFO,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 19, 19);
	return(ret);
}



//*****************************************************************************
static int _ao_output_mode(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_AO_OUTPUT_MODE_IMMED,
		HSDI4AO4_AO_OUTPUT_MODE_SIMUL,
		-1	// terminate list
	};

	int	ret;

	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 18, 18);
	return(ret);
}



//*****************************************************************************
static int _ao_range(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_RANGE_2_5V,
		HSDI4AO4_RANGE_5V,
		HSDI4AO4_RANGE_10V,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 17, 16);
	return(ret);
}



//*****************************************************************************
static int _ao_sw_clock(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x1L << SHIFT)
	#define	SHIFT	20

	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, MASK, MASK);
	return(0);
}



//*****************************************************************************
static int _ao_sw_trigger(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x1L << SHIFT)
	#define	SHIFT	20

	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_boor_32, MASK, MASK);
	return(0);
}



//*****************************************************************************
static int _ao_sync_mode(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_AO_SYNC_MODE_IND,
		HSDI4AO4_AO_SYNC_MODE_AI_SAMP,
		HSDI4AO4_AO_SYNC_MODE_AI_ACQ,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 22, 21);
	return(ret);
}



//*****************************************************************************
static int _data_format(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_DATA_FORMAT_2S_COMP,
		HSDI4AO4_DATA_FORMAT_OFF_BIN,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 25, 25);
	return(ret);
}



//*****************************************************************************
static int _dio_dir_out(dev_data_t* dev, s32* arg)
{
	s32	b0;
	s32	b1;
	int	ret	= 0;

	if (arg[0] == -1)
	{
		b0	= -1;
		gsc_s32_range_reg(dev, &b0, 0, 0x1, dev->vaddr.gsc_diopr_32, 8, 8);

		b1	= -1;
		gsc_s32_range_reg(dev, &b1, 0, 0x1, dev->vaddr.gsc_diopr_32, 24, 24);

		arg[0]	= b0 | (b1 << 1);
	}
	else if ((arg[0] & 0x3) == arg[0])
	{
		b0	= (arg[0] >> 0) & 0x1;
		gsc_s32_range_reg(dev, &b0, 0, 0x1, dev->vaddr.gsc_diopr_32, 8, 8);

		b1	= (arg[0] >> 1) & 0x1;
		gsc_s32_range_reg(dev, &b1, 0, 0x1, dev->vaddr.gsc_diopr_32, 24, 24);
	}
	else
	{
		ret	= EINVAL;
	}

	return(ret);
}



//*****************************************************************************
static int _dio_read(dev_data_t* dev, s32* arg)
{
	s32	b0;
	s32	b1;
	int	ret	= 0;

	b0	= -1;
	gsc_s32_range_reg(dev, &b0, 0, 0xFF, dev->vaddr.gsc_diopr_32, 7, 0);

	b1	= -1;
	gsc_s32_range_reg(dev, &b1, 0, 0xFF, dev->vaddr.gsc_diopr_32, 23, 16);

	arg[0]	= b0 | (b1 << 8);
	return(ret);
}



//*****************************************************************************
static int _dio_write(dev_data_t* dev, s32* arg)
{
	s32	b0;
	s32	b1;
	int	ret	= 0;

	if (arg[0] == -1)
	{
		ret	= _dio_read(dev, arg);
	}
	else if ((arg[0] & 0xFFFF) == arg[0])
	{
		b0	= (arg[0] >> 0) & 0xFF;
		gsc_s32_range_reg(dev, &b0, 0, 0xFF, dev->vaddr.gsc_diopr_32, 7, 0);

		b1	= (arg[0] >> 8) & 0xFF;
		gsc_s32_range_reg(dev, &b1, 0, 0xFF, dev->vaddr.gsc_diopr_32, 23, 16);
	}
	else
	{
		ret	= EINVAL;
	}

	return(ret);
}



//*****************************************************************************
static int _gen_a_enable(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_GEN_ENABLE_NO,
		HSDI4AO4_GEN_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 26, 26);
	return(ret);
}



//*****************************************************************************
static int _gen_a_ndiv(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 20, dev->vaddr.gsc_rgar_32, 28, 24);
	return(ret);
}



//*****************************************************************************
static int _gen_a_nref(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 30, 1000, dev->vaddr.gsc_rgar_32, 21, 12);
	return(ret);
}



//*****************************************************************************
static int _gen_a_nvco(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 30, 1000, dev->vaddr.gsc_rgar_32, 9, 0);
	return(ret);
}



//*****************************************************************************
static int _gen_b_enable(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_GEN_ENABLE_NO,
		HSDI4AO4_GEN_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 27, 27);
	return(ret);
}



//*****************************************************************************
static int _gen_b_ndiv(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 40, 0xFFFFFF, dev->vaddr.gsc_rgbr_32, 23, 0);
	return(ret);
}



//*****************************************************************************
static int _gen_c_enable(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_GEN_ENABLE_NO,
		HSDI4AO4_GEN_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 22, 22);
	return(ret);
}



//*****************************************************************************
static int _gen_c_ndiv(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 40, 0xFFFFFF, dev->vaddr.gsc_rgcr_32, 23, 0);
	return(ret);
}



//*****************************************************************************
static int _master_clock(dev_data_t* dev, s32* arg)
{
	int	ret		= 0;

	if (arg[0] == -1)
	{
		arg[0]	= dev->master_clock;
	}
	else if (	(arg[0] >= HSDI4AO4_MASTER_CLOCK_MIN) &&
				(arg[0] <= HSDI4AO4_MASTER_CLOCK_MAX))
	{
		dev->master_clock	= arg[0];
	}
	else
	{
		ret	= -EINVAL;
	}

	return(ret);
}



//*****************************************************************************
static int _trigger_mode(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_TRIGGER_MODE_IN,
		HSDI4AO4_TRIGGER_MODE_OUT,
		-1	// terminate list
	};

	int	ret;

	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 21, 21);
	return(ret);
}



//*****************************************************************************
static int _rx_io_abort(dev_data_t* dev, s32* arg)
{
	arg[0]	= gsc_read_abort_active_xfer(dev, &dev->io.rx);
	return(0);
}



//*****************************************************************************
static int _tx_io_abort(dev_data_t* dev, s32* arg)
{
	arg[0]	= gsc_write_abort_active_xfer(dev, &dev->io.tx);
	return(0);
}



//*****************************************************************************
static int _master_clock_adj(dev_data_t* dev, s32* arg)
{
	int	ret;

	if (dev->cache.mcar)
		ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFF, dev->vaddr.gsc_mcar_32, 15, 0);
	else
		ret	= -EINVAL;
	return(ret);
}



//*****************************************************************************
static int _ao_buf_empty(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HSDI4AO4_AO_BUFFER_NOT_EMPTY,
		HSDI4AO4_AO_BUFFER_EMPTY,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 12, 12);
	return(ret);
}



//*****************************************************************************
static int _irq_enable(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0x7FFF, dev->vaddr.gsc_psr_32, 14, 0);
	return(ret);
}



// variables ******************************************************************

const gsc_ioctl_t	dev_ioctl_list[]	=
{
	{ HSDI4AO4_IOCTL_REG_READ,			(void*) gsc_reg_read_ioctl		},
	{ HSDI4AO4_IOCTL_REG_WRITE,			(void*) gsc_reg_write_ioctl		},
	{ HSDI4AO4_IOCTL_REG_MOD,			(void*) gsc_reg_mod_ioctl		},
	{ HSDI4AO4_IOCTL_QUERY,				(void*) _query					},
	{ HSDI4AO4_IOCTL_INITIALIZE,		(void*) initialize_ioctl		},
	{ HSDI4AO4_IOCTL_AUTOCAL,			(void*) _autocal				},
	{ HSDI4AO4_IOCTL_AUTOCAL_STATUS,	(void*) _autocal_status			},
	{ HSDI4AO4_IOCTL_RX_IO_ABORT,		(void*) _rx_io_abort			},
	{ HSDI4AO4_IOCTL_RX_IO_MODE,		(void*) _rx_io_mode				},
	{ HSDI4AO4_IOCTL_RX_IO_OVERFLOW,	(void*) _rx_io_overflow			},
	{ HSDI4AO4_IOCTL_RX_IO_TIMEOUT,		(void*) _rx_io_timeout			},
	{ HSDI4AO4_IOCTL_RX_IO_UNDERFLOW,	(void*) _rx_io_underflow		},
	{ HSDI4AO4_IOCTL_TX_IO_ABORT,		(void*) _tx_io_abort			},
	{ HSDI4AO4_IOCTL_TX_IO_MODE,		(void*) _tx_io_mode				},
	{ HSDI4AO4_IOCTL_TX_IO_OVERFLOW,	(void*) _tx_io_overflow			},
	{ HSDI4AO4_IOCTL_TX_IO_TIMEOUT,		(void*) _tx_io_timeout			},
	{ HSDI4AO4_IOCTL_AI_BUF_CLEAR,		(void*) _ai_buf_clear			},
	{ HSDI4AO4_IOCTL_AI_BUF_LEVEL,		(void*) _ai_buf_level			},
	{ HSDI4AO4_IOCTL_AI_BUF_OVERFLOW,	(void*) _ai_buf_overflow		},
	{ HSDI4AO4_IOCTL_AI_BUF_THR_LVL,	(void*) _ai_buf_thr_lvl			},
	{ HSDI4AO4_IOCTL_AI_BUF_THR_STS,	(void*) _ai_buf_thr_sts			},
	{ HSDI4AO4_IOCTL_AI_BUF_UNDERFLOW,	(void*) _ai_buf_underflow		},
	{ HSDI4AO4_IOCTL_AI_BURST_ENABLE,	(void*) _ai_burst_enable		},
	{ HSDI4AO4_IOCTL_AI_BURST_SIZE,		(void*) _ai_burst_size			},
	{ HSDI4AO4_IOCTL_AI_BURST_STATUS,	(void*) _ai_burst_status		},
	{ HSDI4AO4_IOCTL_AI_CHAN_SEL,		(void*) _ai_chan_sel			},
	{ HSDI4AO4_IOCTL_AI_CLOCK_MODE,		(void*) _ai_clock_mode			},
	{ HSDI4AO4_IOCTL_AI_ENABLE,			(void*) _ai_enable				},
	{ HSDI4AO4_IOCTL_AI_MODE,			(void*) _ai_mode				},
	{ HSDI4AO4_IOCTL_AI_RANGE	,		(void*) _ai_range				},
	{ HSDI4AO4_IOCTL_AI_SW_CLOCK,		(void*) _ai_sw_clock			},
	{ HSDI4AO4_IOCTL_AI_SW_TRIGGER,		(void*) _ai_sw_trigger			},
	{ HSDI4AO4_IOCTL_AO_BUF_CLEAR,		(void*) _ao_buf_clear			},
	{ HSDI4AO4_IOCTL_AO_BUF_EMPTY,		(void*) _ao_buf_empty			},
	{ HSDI4AO4_IOCTL_AO_BUF_LEVEL,		(void*) _ao_buf_level			},
	{ HSDI4AO4_IOCTL_AO_BUF_LOAD_REQ,	(void*) _ao_buf_load_req		},
	{ HSDI4AO4_IOCTL_AO_BUF_LOAD_STS,	(void*) _ao_buf_load_sts		},
	{ HSDI4AO4_IOCTL_AO_BUF_MODE,		(void*) _ao_buf_mode			},
	{ HSDI4AO4_IOCTL_AO_BUF_OVER_DATA,	(void*) _ao_buf_over_data		},
	{ HSDI4AO4_IOCTL_AO_BUF_OVER_FRAM,	(void*) _ao_buf_over_fram		},
	{ HSDI4AO4_IOCTL_AO_BUF_THR_LVL,	(void*) _ao_buf_thr_lvl			},
	{ HSDI4AO4_IOCTL_AO_BUF_THR_STS,	(void*) _ao_buf_thr_sts			},
	{ HSDI4AO4_IOCTL_AO_BURST_ENABLE,	(void*) _ao_burst_enable		},
	{ HSDI4AO4_IOCTL_AO_BURST_STATUS,	(void*) _ao_burst_status		},
	{ HSDI4AO4_IOCTL_AO_CHAN_SEL,		(void*) _ao_chan_sel			},
	{ HSDI4AO4_IOCTL_AO_CLOCK_MODE,		(void*) _ao_clock_mode			},
	{ HSDI4AO4_IOCTL_AO_CLOCK_STATUS,	(void*) _ao_clock_status		},
	{ HSDI4AO4_IOCTL_AO_ENABLE,			(void*) _ao_enable				},
	{ HSDI4AO4_IOCTL_AO_MODE,			(void*) _ao_mode				},
	{ HSDI4AO4_IOCTL_AO_OUTPUT_MODE,	(void*) _ao_output_mode			},
	{ HSDI4AO4_IOCTL_AO_RANGE,			(void*) _ao_range				},
	{ HSDI4AO4_IOCTL_AO_SW_CLOCK,		(void*) _ao_sw_clock			},
	{ HSDI4AO4_IOCTL_AO_SW_TRIGGER,		(void*) _ao_sw_trigger			},
	{ HSDI4AO4_IOCTL_AO_SYNC_MODE,		(void*) _ao_sync_mode			},
	{ HSDI4AO4_IOCTL_DATA_FORMAT,		(void*) _data_format			},
	{ HSDI4AO4_IOCTL_DIO_DIR_OUT,		(void*) _dio_dir_out			},
	{ HSDI4AO4_IOCTL_DIO_READ,			(void*) _dio_read				},
	{ HSDI4AO4_IOCTL_DIO_WRITE,			(void*) _dio_write				},
	{ HSDI4AO4_IOCTL_GEN_A_ENABLE,		(void*) _gen_a_enable			},
	{ HSDI4AO4_IOCTL_GEN_A_NDIV,		(void*) _gen_a_ndiv				},
	{ HSDI4AO4_IOCTL_GEN_A_NREF,		(void*) _gen_a_nref				},
	{ HSDI4AO4_IOCTL_GEN_A_NVCO,		(void*) _gen_a_nvco				},
	{ HSDI4AO4_IOCTL_GEN_B_ENABLE,		(void*) _gen_b_enable			},
	{ HSDI4AO4_IOCTL_GEN_B_NDIV,		(void*) _gen_b_ndiv				},
	{ HSDI4AO4_IOCTL_GEN_C_ENABLE,		(void*) _gen_c_enable			},
	{ HSDI4AO4_IOCTL_GEN_C_NDIV,		(void*) _gen_c_ndiv				},
	{ HSDI4AO4_IOCTL_IRQ_ENABLE,		(void*) _irq_enable				},
	{ HSDI4AO4_IOCTL_MASTER_CLOCK,		(void*) _master_clock			},
	{ HSDI4AO4_IOCTL_MASTER_CLOCK_ADJ,	(void*) _master_clock_adj		},
	{ HSDI4AO4_IOCTL_TRIGGER_MODE,		(void*) _trigger_mode			},
	{ HSDI4AO4_IOCTL_WAIT_EVENT,		(void*) gsc_wait_event_ioctl	},
	{ HSDI4AO4_IOCTL_WAIT_CANCEL,		(void*) gsc_wait_cancel_ioctl	},
	{ HSDI4AO4_IOCTL_WAIT_STATUS,		(void*) gsc_wait_status_ioctl	},
	{ -1, NULL }
};


