// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/driver/main.h $
// $Rev: 53435 $
// $Date: 2023-07-24 10:01:26 -0500 (Mon, 24 Jul 2023) $

// 16HSDI4AO4: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "16hsdi4ao4.h"

#define	DEV_BAR_SHOW						0
#define	DEV_METRICS_SHOW					0
#define	DEV_PCI_ID_SHOW						0
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_READ
#define	DEV_SUPPORTS_WAIT
#define	DEV_SUPPORTS_WRITE
#define	GSC_ALT_DEV_GET(a)					(a)
#define	GSC_ALT_STRUCT_GET(i,d)				(d)
#define	GSC_ALT_STRUCT_T					dev_data_t
#define	GSC_DEVS_PER_BOARD					1

typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// macros *********************************************************************

#define	DEV_MODEL							"16HSDI4AO4"	// Upper case form of the below.
#define	DEV_NAME							"16hsdi4ao4"	// MUST AGREE WITH HSDI4AO4_BASE_NAME
#define	DEV_NAME_LC							DEV_NAME
#define	DEV_NAME_UC							DEV_MODEL

#define	DEV_VERSION_STR						GSC_STR(DEV_VERSION_MAJ_NUM) "." GSC_STR(DEV_VERSION_MIN_NUM)
#define	DEV_VERSION							DEV_VERSION_STR

#define	DEV_VERSION_MAJ_NUM					2	// FOR DEVICE SPECIFIC CODE ONLY!
#define	DEV_VERSION_MIN_NUM					8	// FOR DEVICE SPECIFIC CODE ONLY!
// 2.8	Added query options: HSDI4AO4_QUERY_GEN_A_MAX/MIN.
//		Updated the Initialize IOCTL service for consistency.
//		Updated the Autocal IOCTL service for consistency.
//		Changed spelling of all AUTO_CAL content to AUTOCAL (case insensitive).
//		Corrected error message text when Clear Output Buffer takes too long.
//		Updated makefile for consistency.
// 2.7	Updated to support the 6.x series kernel.
//		Updated comments for consistency.
//		Added new query options.
//		Updated the Initialize and Autocal IOCTL services for consistency.
//		Removed unused macros.
//		Updated per PLX code segregation.
// 2.6	Removed compiler warnings for Fedora 31.
//		Updated initial file content for consistency.
//		Updated for porting to Windows.
//		Updated I/O availability calculations.
//		Standardized various IOCTL system log messages.
//		Standardized processing of the initialization IOCTL service.
//		Standardized processing of the autocal IOCTL service.
// 2.5	Added support for the 5.x kernel series.
//		Added Endianness support.
//		Added some IRQ comments.
//		Some whitespace cleanup.
// 2.4	Updated per changes to the common I/O code.
//		Corrected a Demand Mode DMA bug introduced in version 2.3.
// 2.3	Bug fix: DMDMA writes produced corrupted output.
// 2.2	Bug fix: was ignoring BAR setup return value.
//		Updated per changes to the OS specific PCI services.
//		Made updates for DEV_SUPPORTS_XXX macro changes.
//		Modified code so open succeeds even if initialization fails.
//		Bug fix in HSDI4AO4_IOCTL_AI_MODE service.
// 2.1	The BAR code has been updated to include the data under a single structure.
//		The register definitions have been updated.
//		I/O services now pass around an os_mem_t structure instead of a buffer pointer.
//		White space cleanup.
//		Changed the arbitrary wait event callback argument to a void* type.// 2.0	Updated to use the newer common driver sources.
//		Modified to allow multiple apps to simultaneously access a single device.
//		Correct access to BAR0 and BAR1 (is RO, was RW).
//		Added an infinite I/O timeout option.
//		Corrected the IOCTL code for the register write and mod services.
// 2.0	Updated to use the newer common driver sources.
//		Removed GNU notice from non-Linux specific files.
//		Removed Linux specific content from non-Linux specific source files.
//		White space cleanup.
//		Now using a spinlock rather than enabling and disabling interrupts.
//		Updated gsc_irq_open() and gsc_irq_close().
//		Updated gsc_dma_open() and gsc_dma_close().
// 1.5	Reduced #include list in driver interface header.
// 1.4	Updated for the 3.x kernel.
//		Corrected a system timer count roll over bug in ioctl.c.
// 1.3	BAR0 and BAR2 are now the only BAR regions used.
//		Include all common source, though not all are used.
// 1.2	Eliminated the global dev_check_id() routine.
// 1.1	Updated per changes to the common source code.
//		Implemented IOCTL based I/O Abort support.
//		Implemented IOCTL based event wait support.
//		Updated to use the common gsc_open() and gsc_close() services.
//		Implemented support for the common PIO I/O routines.
//		Changed use of DEV_SUPPORTS_READ macro.
//		Changed use of DEV_SUPPORTS_WRITE macro.
// 1.0	First official release.
// BETA.0	Preliminary release of the new driver.

#ifndef	TRUE
	#define	TRUE							1
#endif

#ifndef	FALSE
	#define	FALSE							0
#endif

// I/O services
#define	DEV_IO_STREAM_QTY					(DEV_IO_RX_STREAM_QTY + DEV_IO_TX_STREAM_QTY)
#define	DEV_IO_RX_STREAM_QTY				1
#define	DEV_IO_TX_STREAM_QTY				1

#define	GSC_READ_PIO_WORK_32_BIT
#define	GSC_WRITE_PIO_WORK_32_BIT

// WAIT services
#define	DEV_WAIT_GSC_ALL					HSDI4AO4_WAIT_GSC_ALL
#define	DEV_WAIT_ALT_ALL					HSDI4AO4_WAIT_ALT_ALL
#define	DEV_WAIT_IO_ALL						HSDI4AO4_WAIT_IO_ALL



// data types *****************************************************************

struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).

	s32					io_mode;			// PIO, BMDMA, DMDMA
	s32					overflow_check;		// Check for overflows?
	s32					pio_threshold;		// Use PIO if samples <= this.
	s32					timeout_s;			// I/O timeout in seconds.
	s32					underflow_check;	// Check for underflows?

	// Initialized by I/O service at time of use.

	int					abort;				// Also set via IOCTL service.
	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	int					non_blocking;		// Is this non-blocking I/O?

	// Initialized by device specific I/O create code.

	int					bytes_per_sample;	// Sample size in bytes.
	u32					io_reg_offset;		// Offset of device's I/O FIFO.
	VADDR_T				io_reg_vaddr;		// Address of device's I/O FIFO.

	void				(*dev_io_sw_init)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_close)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_open)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	int					(*dev_io_startup)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	long				(*dev_pio_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_bmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_dmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_pio_xfer)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_bmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_dmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

	struct
	{
		u32				abort;		// ...WAIT_IO_XXX_ABORT
		u32				done;		// ...WAIT_IO_XXX_DONE
		u32				error;		// ...WAIT_IO_XXX_ERROR
		u32				timeout;	// ...WAIT_IO_XXX_TIMEOUT
	} wait;

	// Initialize by GSC commone create code.

	os_mem_t			mem;				// I/O buffer.
	os_sem_t			sem;				// Only one Tx or Rx at a time.
};

struct _dev_data_t
{
	// Put an integral data type at beginning of structure.
	int					board_index;	// Index of the device being accessed.
	gsc_dev_type_t		board_type;		// Corresponds to basic device type.
	s32					master_clock;	// Master clock frequency
	const char*			model;			// Base model number in upper case.
	int					share;			// Were we opened in shared mode?
	int					users;			// Number of currently active open() requests.

	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.

	gsc_bar_t			bar;			// device register mappings
	gsc_dma_t			dma;			// For DMA based I/O.
	gsc_irq_t			irq;			// For interrupts.
	gsc_wait_node_t*	wait_list;

	struct
	{					// This is for streaming I/O
		dev_io_t		rx;				// For read operations.
		dev_io_t		tx;				// For write operations.
		dev_io_t*		io_streams[DEV_IO_STREAM_QTY];
	} io;

	struct
	{
		VADDR_T			plx_intcsr_32;	// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;	// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;	// DMA Threshold Register

		VADDR_T			gsc_acr_32;		// Assembly Configuration Register
		VADDR_T			gsc_aibr_32;	// Analog Input Buffer Register
		VADDR_T			gsc_aobr_32;	// Analog Output Buffer Register
		VADDR_T			gsc_bcr_32;		// Board Control Register
		VADDR_T			gsc_boor_32;	// Buffered Output Operations Register
		VADDR_T			gsc_diopr_32;	// Digital I/O Port Register
		VADDR_T			gsc_ibsr_32;	// Input Buffer Size Register
		VADDR_T			gsc_ibtr_32;	// Input Buffer Threshold Register
		VADDR_T			gsc_icr_32;		// Input Configuration Register
		VADDR_T			gsc_mcar_32;	// Master Clock Adjust Register
		VADDR_T			gsc_obsr_32;	// Ouput Buffer Size Register
		VADDR_T			gsc_obtr_32;	// Output Buffer Threshold Register
		VADDR_T			gsc_psr_32;		// Primary Status Register
		VADDR_T			gsc_rgar_32;	// Rate Generator A Register
		VADDR_T			gsc_rgbr_32;	// Rate Generator B Register
		VADDR_T			gsc_rgcr_32;	// Rate Generator C Register

	} vaddr;

	struct
	{
		s32				ai_chans_max;		// Maximum input channels supported.
		s32				ai_chan_qty;		// The number of input channels on the device.
		s32				ao_chans_max;		// Maximum output channels supported.
		s32				ao_chan_qty;		// The number of output channels on the device.
		s32				autocal_ms;			// Maximum ms for autocalibration

		u32				fifo_size_rx;		// Size of Rx FIFO - not the fill level.
		u32				fifo_size_tx;		// Size of Tx FIFO - not the fill level.
		s32				fsamp_max;			// The maximum Fsamp rate per channel.
		s32				fsamp_min;			// The minimum Fsamp rate per channel.

		s32				gen_a_max;			// Generator A maximum
		s32				gen_a_min;			// Generator A minimum
		s32				gen_a_ndiv_max;		// Generator A Ndiv maximum
		s32				gen_a_ndiv_min;		// Generator A Ndiv minimum
		s32				gen_a_nref_max;		// Generator A Nref maximum
		s32				gen_a_nref_min;		// Generator A Nref minimum
		s32				gen_a_nvco_max;		// Generator A Nvco maximum
		s32				gen_a_nvco_min;		// Generator A Nvco minimum
		s32				gen_b_ndiv_max;		// Generator B Ndiv maximum
		s32				gen_b_ndiv_min;		// Generator B Ndiv minimum
		s32				gen_c_ndiv_max;		// Generator C Ndiv maximum
		s32				gen_c_ndiv_min;		// Generator C Ndiv minimum
		u32				gsc_acr_32;			// Assembly Configuration Register

		s32				initialize_ms;		// Maximum ms for initialize

		s32				master_clock_def;	// Default master clock frequency
		s32				mcar;				// Is this register available?

	} cache;
};



// prototypes *****************************************************************

void		dev_io_close(dev_data_t* dev);
int			dev_io_create(dev_data_t* dev);
void		dev_io_destroy(dev_data_t* dev);
int			dev_io_open(dev_data_t* dev);
dev_io_t*	dev_io_read_select(dev_data_t* dev, size_t bytes);
dev_io_t*	dev_io_write_select(dev_data_t* dev, size_t bytes);
int			dev_irq_create(dev_data_t* dev);
void		dev_irq_destroy(dev_data_t* dev);

int			dev_read_create(dev_data_t* dev, dev_io_t* io);

int			dev_write_create(dev_data_t* dev, dev_io_t* io);

int			initialize_ioctl(dev_data_t* dev, void* arg);



#endif
