// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/samples/fsamp/perform.c $
// $Rev: 53426 $
// $Date: 2023-07-20 12:59:40 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	FILE_NAME				"fsamp.txt"
#define	FILE_NAME_LN			"fsamp_ln.txt"
#define	LIMIT_MAX(v,max)		(((v) > (max)) ? (max) : (v))
#define	LIMIT_MIN(v,min)		(((v) < (min)) ? (min) : (v))
#define	LIMIT_RANGE(v,min,max)	(LIMIT_MIN((LIMIT_MAX((v),(max))),(min)))



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int			errs	= 0;
	FILE*		file	= NULL;
	s32			fsamp;
	s32			max;
	s32			min;
	const char*	name;
	double		rate;

	gsc_label("Fsamp Computation");
	printf("\n");
	gsc_label_level_inc();

	errs	+= hsdi4ao4_query(args->fd, -1, 1, HSDI4AO4_QUERY_FSAMP_MAX, &max);
	errs	+= hsdi4ao4_query(args->fd, -1, 1, HSDI4AO4_QUERY_FSAMP_MIN, &min);

	if (args->range == 0)
	{
		min	= LIMIT_RANGE(args->fsamp, min, max);
		max	= min;
	}

	if (args->save)
	{
		name	= (args->low_noise) ? FILE_NAME_LN : FILE_NAME;
		file	= fopen(name, "w+b");

		if (file == NULL)
		{
			printf("FAIL <---  (unable to create %s)\n", name);
			errs	= 1;
		}
		else
		{
			fprintf(file, "# 16HSDI4AO4 Scan Rates\n");
			fprintf(file, "# Request  Produced\n");
			fprintf(file, "# =======  ==========\n");
		}
	}

	for (fsamp = min; fsamp <= max; fsamp++)
	{
		errs	+= hsdi4ao4_fsamp_ai_compute(args->fd, -1, 1, fsamp, args->low_noise, NULL, NULL, NULL, &rate);

		if (file)
			fprintf(file, "%9ld  %10.3f\n", (long) fsamp, (float) rate);
	}

	if (file)
		fclose(file);

	gsc_label_level_dec();
	return(errs);
}


