// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/samples/id/perform.c $
// $Rev: 53421 $
// $Date: 2023-07-20 12:50:00 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static void _input_modes(void)
{
	gsc_label("Input Modes");
	printf("Differential, Single-Ended, Zero Test, +Vref Test, Loopback\n");
}



//*****************************************************************************
static void _voltage_ranges(void)
{
	gsc_label("Voltage Ranges");
	printf("+-10V, +-5V, +-2.5V\n");
}



//*****************************************************************************
static void _rate_generators(void)
{
	gsc_label("Rate Generators");
	printf("3  (A-C)\n");
}



//*****************************************************************************
static void _clocking_sources(void)
{
	gsc_label("Clocking Sources");
	printf("Rate Generator(s), External, Software\n");
}



//*****************************************************************************
static void _sampling_modes(void)
{
	gsc_label("Input Sampling Modes");
	printf("Continuous, Burst\n");

	gsc_label("Output Sampling Modes");
	printf("Continuous, Burst\n");

	gsc_label("Output Update Modes");
	printf("Immediate, Synchronous\n");

	gsc_label("Output Modes");
	printf("Buffered, Unbuffered\n");

	gsc_label("Output Buffer Modes");
	printf("Open, Closed (recycle)\n");
}



//*****************************************************************************
static void _digital_io(void)
{
	gsc_label("Digital I/O");
	printf("16 Signals (2x byte wide)\n");
}



//*****************************************************************************
static int _id_device_reg(const args_t* args)
{
	int						errs;
	gsc_reg_def_t			list[2]	= { { NULL }, { NULL } };
	const gsc_reg_def_t*	ptr;

	printf("\n");
	ptr	= hsdi4ao4_reg_get_def_id(HSDI4AO4_GSC_ACR);

	if (ptr)
	{
		list[0]	= ptr[0];
		errs	= gsc_reg_list(args->fd, list, 1, hsdi4ao4_reg_read);
	}
	else
	{
		errs	= 1;
		printf(	"FAIL <---  (%d. hsdi4ao4_reg_get_def_id, HSDI4AO4_GSC_ACR)\n",
				__LINE__);
	}

	return(errs);
}



//*****************************************************************************
static int _register_map(const args_t* args)
{
	int	errs;

	printf("\n");
	errs	= hsdi4ao4_reg_list(args->fd, args->detail);
	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;
	int	i;

	gsc_label("Device Features");
	printf("\n");

	gsc_label_level_inc();

	_input_modes();
	_voltage_ranges();
	_rate_generators();
	_clocking_sources();
	_sampling_modes();
	_digital_io();

	for (i = 0; i < HSDI4AO4_IOCTL_QUERY_LAST; i++)
		errs	+= hsdi4ao4_query(args->fd, -1, 1, i, NULL);

	gsc_label_level_dec();

	errs	+= _id_device_reg(args);
	errs	+= _register_map(args);
	printf("\n");

	return(errs);
}


