// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/samples/sbtest/ai_buf_thr_sts.c $
// $Rev: 53428 $
// $Date: 2023-07-20 13:03:43 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_CLOCK_MODE,
			/* arg		*/	HSDI4AO4_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_GEN_A_ENABLE,
			/* arg		*/	HSDI4AO4_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_CHAN_SEL,
			/* arg		*/	0x1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BURST_SIZE,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BURST_ENABLE,
			/* arg		*/	HSDI4AO4_BURST_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Verify that the FIFO is empty.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSDI4AO4_GSC_IBSR,
			/* mask		*/	0x7FFFF,
			/* value	*/	0x00000
		},
		{					// Set and test the threshold level.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BUF_THR_LVL,
			/* arg		*/	0,
			/* reg		*/	HSDI4AO4_GSC_IBTR,
			/* mask		*/	0x80000,
			/* value	*/	0x00000
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		// Test with 100 samples.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_SW_TRIGGER,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Verify that the FIFO has samples.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSDI4AO4_GSC_IBSR,
			/* mask		*/	0x7FFFF,
			/* value	*/	0x00064
		},
		{					// Set and test the threshold level.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BUF_THR_LVL,
			/* arg		*/	100,
			/* reg		*/	HSDI4AO4_GSC_IBTR,
			/* mask		*/	0x80000,
			/* value	*/	0x00000
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_3[]	=
	{
		{					// Set and test the threshold level.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BUF_THR_LVL,
			/* arg		*/	99,
			/* reg		*/	HSDI4AO4_GSC_IBTR,
			/* mask		*/	0x80000,
			/* value	*/	0x80000
		},


		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_4[]	=
	{
		// Remove one sample then verify the results.

		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSDI4AO4_GSC_AIBR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSDI4AO4_GSC_IBTR,
			/* mask		*/	0x80000,
			/* value	*/	0x00000
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;
	u32	sts;

	errs	+= hsdi4ao4_initialize(fd, -1, 0);

	// Verify that the threshold status is CLEAR.
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list_1);

	ret	= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_AI_BUF_THR_STS, &sts);

	if (ret)
	{
		errs++;
	}
	else if (sts != HSDI4AO4_BUF_THR_STS_CLEAR)
	{
			errs++;
			printf(	"FAIL <---  (%d. expected %ld, got %ld)\n",
					__LINE__,
					(long) HSDI4AO4_BUF_THR_STS_CLEAR,
					(long) sts);
	}

	// Verify that the threshold status is CLEAR.
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list_2);

	ret	= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_AI_BUF_THR_STS, &sts);

	if (ret)
	{
		errs++;
	}
	else if (sts != HSDI4AO4_BUF_THR_STS_CLEAR)
	{
			errs++;
			printf(	"FAIL <---  (%d. expected %ld, got %ld)\n",
					__LINE__,
					(long) HSDI4AO4_BUF_THR_STS_CLEAR,
					(long) sts);
	}

	// Verify that the threshold status is SET.
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list_3);

	ret	= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_AI_BUF_THR_STS, &sts);

	if (ret)
	{
		errs++;
	}
	else if (sts != HSDI4AO4_BUF_THR_STS_SET)
	{
			errs++;
			printf(	"FAIL <---  (%d. expected %ld, got %ld)\n",
					__LINE__,
					(long) HSDI4AO4_BUF_THR_STS_SET,
					(long) sts);
	}

	// Verify that the threshold status is CLEAR.
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list_4);

	ret	= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_AI_BUF_THR_STS, &sts);

	if (ret)
	{
		errs++;
	}
	else if (sts != HSDI4AO4_BUF_THR_STS_CLEAR)
	{
			errs++;
			printf(	"FAIL <---  (%d. expected %ld, got %ld)\n",
					__LINE__,
					(long) HSDI4AO4_BUF_THR_STS_CLEAR,
					(long) sts);
	}

	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ai_buf_thr_sts_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSDI4AO4_IOCTL_AI_BUF_THR_STS.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai_buf_thr_sts_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSDI4AO4_IOCTL_AI_BUF_THR_STS");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


