// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/samples/sbtest/ai_buf_underflow.c $
// $Rev: 53428 $
// $Date: 2023-07-20 13:03:43 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	#undef	MASK
	#define	MASK	0x800000

	service_data_t	list_set[]	=
	{
		{
			/* service	*/	SERVICE_IOCTL_SET,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_IOCTL_SET,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_READ,
			/* cmd		*/	-1,
			/* arg		*/	-1,
			/* reg		*/	HSDI4AO4_GSC_AIBR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	-1,
			/* arg		*/	-1,
			/* reg		*/	HSDI4AO4_GSC_BCR,
			/* mask		*/	MASK,
			/* value	*/	MASK
		},

		{ SERVICE_END_LIST }
	};

	service_data_t	list_clear[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BUF_UNDERFLOW,
			/* arg		*/	HSDI4AO4_BUF_UNDERFLOW_CLEAR,
			/* reg		*/	HSDI4AO4_GSC_BCR,
			/* mask		*/	MASK,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	s32	val;
	int	ret;

	errs	+= hsdi4ao4_initialize(fd, -1, 0);

	for (;;)	// A convenience loop.
	{
		// Verify that there isn't an underflow already.
		val	= HSDI4AO4_BUF_UNDERFLOW_TEST;
		ret	= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_AI_BUF_UNDERFLOW, &val);

		if (ret)
		{
			errs	= 1;
			printf("FAIL <---  (hsdi4ao4_ioctl() error, error %d)\n", ret);
			break;
		}
		else if (val == HSDI4AO4_BUF_UNDERFLOW_YES)
		{
			errs	= 1;
			printf("FAIL <---  (underflow already occurred.)\n");
			break;
		}

		// Now induce an underflow.
		errs	= service_ioctl_reg_get_list(fd, __LINE__, list_set);

		if (errs)
			break;

		// Verify that there is an underflow already.
		val	= HSDI4AO4_BUF_UNDERFLOW_TEST;
		ret	= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_AI_BUF_UNDERFLOW, &val);

		if (ret)
		{
			errs	= 1;
			printf("FAIL <---  (hsdi4ao4_ioctl() error, error %d)\n", ret);
			break;
		}
		else if (val == HSDI4AO4_BUF_UNDERFLOW_NO)
		{
			errs	= 1;
			printf("FAIL <---  (underflow not detected.)\n");
			break;
		}

		// Now clear the underflow.
		errs	= service_ioctl_set_reg_list(fd, __LINE__, list_clear);
		break;
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	ai_buf_underflow_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSDI4AO4_IOCTL_AI_BUF_UNDERFLOW.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai_buf_underflow_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSDI4AO4_IOCTL_AI_BUF_UNDERFLOW");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


