// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/samples/sbtest/ai_burst_size.c $
// $Rev: 53428 $
// $Date: 2023-07-20 13:03:43 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BURST_SIZE,
			/* arg		*/	0x00000000,
			/* reg		*/	HSDI4AO4_GSC_ICR,
			/* mask		*/	0x00FFFFFF,
			/* value	*/	0x00000000
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;
	int	i;

	errs	+= hsdi4ao4_initialize(fd, -1, 0);

	for (i = -1; i <= 24; i++)
	{
		list[0].arg		= (0x1L << i) & 0xFFFFFF;
		list[0].value	= list[0].arg;

		errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);
		errs	+= service_ioctl_reg_get_list(fd, __LINE__, list);

		list[0].arg		= (~list[0].arg) & 0xFFFFFF;
		list[0].value	= list[0].arg;

		errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);
		errs	+= service_ioctl_reg_get_list(fd, __LINE__, list);
	}

	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	service_data_t	list[]	=
	{
		{	// 0			// Initialize the device.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 1			// Enable bursting.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BURST_ENABLE,
			/* arg		*/	HSDI4AO4_BURST_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 2			// Set the burst size.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BURST_SIZE,
			/* arg		*/	0,	// SET THIS FOR EACH LOOP ITERATION.
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 3			// Scan a single channel only.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_CHAN_SEL,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 4
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_CLOCK_MODE,
			/* arg		*/	HSDI4AO4_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 5			// Enable Rate-A Generator
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_GEN_A_ENABLE,
			/* arg		*/	HSDI4AO4_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 6			// Disable Rate-B Generator
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_GEN_B_ENABLE,
			/* arg		*/	HSDI4AO4_GEN_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 7		// Clear the buffer.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 8		// Enable input clocking.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 9			// Initiate a trigger.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_SW_TRIGGER,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 10			// Wait for possible data reception.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 11			// Verify the amount of data.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSDI4AO4_GSC_IBSR,
			/* mask		*/	0x7FFFF,
			/* value	*/	0	// SET IN EACH LOOP ITERATION
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	i;

	errs	+= hsdi4ao4_initialize(fd, -1, 0);

	// We're only testing the level accuracy for a small range.

	for (i = 5; (errs == 0) && (i < 10); i++)
	{
		list[2].arg		= 111 * i;
		list[11].value	= 111 * i;
		errs			+= service_ioctl_set_reg_list(fd, __LINE__, list);
	}

	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ai_burst_size_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSDI4AO4_IOCTL_AI_BURST_SIZE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai_burst_size_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSDI4AO4_IOCTL_AI_BURST_SIZE");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


