// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/samples/sbtest/ao_buf_clear.c $
// $Rev: 53428 $
// $Date: 2023-07-20 13:03:43 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}



//*****************************************************************************
static int _tx_fifo_overflow(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_MODE,
			/* arg		*/	HSDI4AO4_AO_MODE_FIFO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HSDI4AO4_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	boor;
	int	errs	= 0;
	int	i;

	for (i = 0; i < 1024; i++)
	{
		errs	+= service_ioctl_set_list(fd, __LINE__, list);
		errs	+= hsdi4ao4_reg_read(fd, -1, 0, HSDI4AO4_GSC_BOOR, &boor);

		if (boor & 0x10000)
		{
			// The Tx FIFO is not FULL
			continue;
		}
		else
		{
			// The Tx FIFO is FULL.
			break;
		}
	}

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Verify that we got an overflow.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSDI4AO4_GSC_BOOR,
			/* mask		*/	0x10000,
			/* value	*/	0x00000
		},

		// Clear the buffer and verify the results.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	-1,
			/* arg		*/	0,
			/* reg		*/	HSDI4AO4_GSC_BOOR,
			/* mask		*/	0x10000,
			/* value	*/	0x00000
		},
		{
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	-1,
			/* arg		*/	0,
			/* reg		*/	HSDI4AO4_GSC_OBSR,
			/* mask		*/	0x7FFFF,
			/* value	*/	0x00000
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	errs	+= _tx_fifo_overflow(fd);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);
	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ao_buf_clear_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSDI4AO4_IOCTL_AO_BUF_CLEAR.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao_buf_clear_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSDI4AO4_IOCTL_AO_BUF_CLEAR");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


