// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/samples/sbtest/ao_buf_over_fram.c $
// $Rev: 53428 $
// $Date: 2023-07-20 13:03:43 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_MODE,
			/* arg		*/	HSDI4AO4_AO_MODE_FIFO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_BUF_MODE,
			/* arg		*/	HSDI4AO4_BUF_MODE_CIRC,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{					// Force a buffer overflow.
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSDI4AO4_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;
	s32	val;

	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list_1);

	val		= -1;
	ret		= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_AO_BUF_OVER_FRAM, &val);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (val != HSDI4AO4_BUF_OVERFLOW_NO))
	{
		errs++;
		printf(	"FAIL <---  (%d. There is an unexpected overflow.)\n",
				__LINE__);
	}

	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list_2);

	val		= -1;
	ret		= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_AO_BUF_OVER_FRAM, &val);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (val != HSDI4AO4_BUF_OVERFLOW_YES))
	{
		errs++;
		printf(	"FAIL <---  (%d. Expected overflow did not occur.)\n",
				__LINE__);
	}

	val		= HSDI4AO4_BUF_OVERFLOW_TEST;
	ret		= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_AO_BUF_OVER_FRAM, &val);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (val != HSDI4AO4_BUF_OVERFLOW_YES))
	{
		errs++;
		printf(	"FAIL <---  (%d. An overflow was unexpectedly cleared.)\n",
				__LINE__);
	}

	val		= HSDI4AO4_BUF_OVERFLOW_CLEAR;
	ret		= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_AO_BUF_OVER_FRAM, &val);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (val != HSDI4AO4_BUF_OVERFLOW_NO))
	{
		errs++;
		printf(	"FAIL <---  (%d. An overflow was not cleared.)\n",
				__LINE__);
	}

	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ao_buf_over_fram_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSDI4AO4_IOCTL_AO_BUF_OVER_FRAM.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao_buf_over_fram_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSDI4AO4_IOCTL_AO_BUF_OVER_FRAM");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


