// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/samples/sbtest/ao_buf_thr_sts.c $
// $Rev: 53428 $
// $Date: 2023-07-20 13:03:43 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	// This is tested below.
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_MODE,
			/* arg		*/	HSDI4AO4_AO_MODE_FIFO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	256,
			/* reg		*/	HSDI4AO4_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_BUF_THR_LVL,
			/* arg		*/	256,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	HSDI4AO4_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_3[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// wait for propogation
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;
	s32	sts;

	errs	+= hsdi4ao4_initialize(fd, -1, 0);

	// Verify that the threshold status is CLEAR.
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list_1);
	ret		= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_AO_BUF_THR_STS, &sts);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (sts != HSDI4AO4_BUF_THR_STS_CLEAR))
	{
			errs++;
			printf(	"FAIL <---  (%d. expected %ld, got %ld)\n",
					__LINE__,
					(long) HSDI4AO4_BUF_THR_STS_CLEAR,
					(long) sts);
	}

	// Verify that the threshold status is SET.
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list_2);
	ret		= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_AO_BUF_THR_STS, &sts);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (sts != HSDI4AO4_BUF_THR_STS_SET))
	{
			errs++;
			printf(	"FAIL <---  (%d. expected %ld, got %ld)\n",
					__LINE__,
					(long) HSDI4AO4_BUF_THR_STS_SET,
					(long) sts);
	}

	// Verify that the threshold status is CLEAR.
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list_3);
	ret		= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_AO_BUF_THR_STS, &sts);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (sts != HSDI4AO4_BUF_THR_STS_CLEAR))
	{
			errs++;
			printf(	"FAIL <---  (%d. expected %ld, got %ld)\n",
					__LINE__,
					(long) HSDI4AO4_BUF_THR_STS_CLEAR,
					(long) sts);
	}

	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ao_buf_thr_sts_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSDI4AO4_IOCTL_AO_BUF_THR_STS.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao_buf_thr_sts_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSDI4AO4_IOCTL_AO_BUF_THR_STS");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


