// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/samples/sbtest/ao_clock_status.c $
// $Rev: 53428 $
// $Date: 2023-07-20 13:03:43 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no static bits to test.
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_CLOCK_MODE,
			/* arg		*/	HSDI4AO4_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_MODE,
			/* arg		*/	HSDI4AO4_AO_MODE_FIFO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	static const service_data_t	list_2[]	=
	{
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1,
			/* reg		*/	HSDI4AO4_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;
	s32	status;

	errs	+= hsdi4ao4_initialize(fd, -1, 0);

	// Verify NOT READY.
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list_1);

	ret		= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_AO_CLOCK_STATUS, &status);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (status != HSDI4AO4_CLOCK_STATUS_NOT_READY))
	{
		errs++;
		printf("FAIL <---  (%d. Should NOT be READY, but it is.)\n", __LINE__);
	}

	// Verify READY.
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list_2);

	ret		= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_AO_CLOCK_STATUS, &status);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (status != HSDI4AO4_CLOCK_STATUS_READY))
	{
		errs++;
		printf("FAIL <---  (%d. Should be READY, but it isn't.)\n", __LINE__);
	}

	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ao_clock_status_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSDI4AO4_IOCTL_AO_CLOCK_STATUS.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao_clock_status_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSDI4AO4_IOCTL_AO_CLOCK_STATUS");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


