// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/samples/sbtest/ao_enable.c $
// $Rev: 53428 $
// $Date: 2023-07-20 13:03:43 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_YES,
			/* reg		*/	HSDI4AO4_GSC_BOOR,
			/* mask		*/	0x20,
			/* value	*/	0x20
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_NO,
			/* reg		*/	HSDI4AO4_GSC_BOOR,
			/* mask		*/	0x20,
			/* value	*/	0x00
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= hsdi4ao4_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);
	errs	+= service_ioctl_reg_get_list(fd, __LINE__, list);

	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);
	errs	+= service_ioctl_reg_get_list(fd, __LINE__, list);

	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_CLOCK_MODE,
			/* arg		*/	HSDI4AO4_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_MODE,
			/* arg		*/	HSDI4AO4_AO_MODE_FIFO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_GEN_C_NDIV,
			/* arg		*/	0xFF,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_GEN_C_ENABLE,
			/* arg		*/	HSDI4AO4_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Verify the DISABLE option.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Data.
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSDI4AO4_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0x0000
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	20,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Verify that the buffer has 1 value.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSDI4AO4_GSC_OBSR,
			/* mask		*/	0x7FFFF,
			/* value	*/	0x00001
		},

		// Verify the ENABLE option.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Verify that the buffer is empty.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSDI4AO4_GSC_OBSR,
			/* mask		*/	0x7FFFF,
			/* value	*/	0x00000
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);
	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	ao_enable_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSDI4AO4_IOCTL_AO_ENABLE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao_enable_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSDI4AO4_IOCTL_AO_ENABLE");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


