// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/samples/sbtest/ao_range.c $
// $Rev: 53428 $
// $Date: 2023-07-20 13:03:43 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_RANGE,
			/* arg		*/	HSDI4AO4_RANGE_2_5V,
			/* reg		*/	HSDI4AO4_GSC_BCR,
			/* mask		*/	0x30000,
			/* value	*/	0x00000
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_RANGE,
			/* arg		*/	HSDI4AO4_RANGE_5V,
			/* reg		*/	HSDI4AO4_GSC_BCR,
			/* mask		*/	0x30000,
			/* value	*/	0x10000
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_RANGE,
			/* arg		*/	HSDI4AO4_RANGE_10V,
			/* reg		*/	HSDI4AO4_GSC_BCR,
			/* mask		*/	0x30000,
			/* value	*/	0x20000
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= hsdi4ao4_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);
	errs	+= service_ioctl_reg_get_list(fd, __LINE__, list);

	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);
	errs	+= service_ioctl_reg_get_list(fd, __LINE__, list);

	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _common_config(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_DATA_FORMAT,
			/* arg		*/	HSDI4AO4_DATA_FORMAT_OFF_BIN,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Configure the outputs.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_MODE,
			/* arg		*/	HSDI4AO4_AO_MODE_REG,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_OUTPUT_MODE,
			/* arg		*/	HSDI4AO4_AO_OUTPUT_MODE_IMMED,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Configure the inputs.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_RANGE,
			/* arg		*/	HSDI4AO4_RANGE_10V,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BURST_ENABLE,
			/* arg		*/	HSDI4AO4_BURST_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BURST_SIZE,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_CLOCK_MODE,
			/* arg		*/	HSDI4AO4_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_GEN_A_ENABLE,
			/* arg		*/	HSDI4AO4_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BUF_THR_LVL,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);
	return(errs);
}



//*****************************************************************************
static int _volt_test(int fd, const char* name, s32 range, s32 value, s32 min, s32 max)
{
	service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_RANGE,
			/* arg		*/	range,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AUTOCAL,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_MODE,
			/* arg		*/	HSDI4AO4_AI_MODE_AO_0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	5,
			/* reg		*/	HSDI4AO4_GSC_AOC0R,
			/* mask		*/	0,
			/* value	*/	value
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_SW_TRIGGER,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Verify that the FIFO has data.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSDI4AO4_GSC_IBTR,
			/* mask		*/	0x80000,
			/* value	*/	0x80000
		},

		{ SERVICE_END_LIST }
	};


	int			errs	= 0;
	gsc_reg_t	parm;
	int			ret;

	errs		+= hsdi4ao4_initialize(fd, -1, 0);
	errs		+= _common_config(fd);
	errs		+= service_ioctl_set_reg_list(fd, __LINE__, list);

	parm.reg	= HSDI4AO4_GSC_AIBR;
	parm.value	= 0xDEADBEEF;
	parm.mask	= 0xBEEFDEAD;
	ret			= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_REG_READ, (void*) &parm);
	value		= parm.value & 0xFFFF;

	if (ret)
	{
		errs++;
		printf(	"FAIL <---  (%d. %s V, hsdi4ao4_ioctl() failure, error %d)\n",
				__LINE__,
				name,
				ret);
	}
	else if ((value < min) || (value > max))
	{
		errs++;
		printf(	"FAIL <---  (%d. %s V range error:"
				" got 0x%lX, min 0x%lX, max 0x%04lX)\n",
				__LINE__,
				name,
				(long) value,
				(long) min,
				(long) max);
	}

	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	#define	_2_5V_OUT		0xFE76				// 2.47 volts
	#define	_2_5V_IN_MAX	(0x9F0D + 0x0FFF)
	#define	_2_5V_IN_MIN	(0x9F0D - 0x0FFF)

	#define	_5V_OUT			0xFE76				// 4.94 volts
	#define	_5V_IN_MAX		(0xBF63 + 0x0FFF)
	#define	_5V_IN_MIN		(0xBF63 - 0x0FFF)

	#define	_10V_OUT		0xFE76				// 9.88 volts
	#define	_10V_IN_MAX		((s32) 0xFE76 + 0x0FFF)
	#define	_10V_IN_MIN		((s32) 0xFE76 - 0x0FFF)

	int	errs	= 0;

	errs	+= _volt_test(fd, "2.5",	HSDI4AO4_RANGE_2_5V,	_2_5V_OUT,	_2_5V_IN_MIN,	_2_5V_IN_MAX);
	errs	+= _volt_test(fd, "5",		HSDI4AO4_RANGE_5V,		_5V_OUT,	_5V_IN_MIN,		_5V_IN_MAX);
	errs	+= _volt_test(fd, "10",		HSDI4AO4_RANGE_10V,		_10V_OUT,	_10V_IN_MIN,	_10V_IN_MAX);

	return(errs);
}



/******************************************************************************
*
*	Function:	ao_range_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSDI4AO4_IOCTL_AO_RANGE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao_range_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSDI4AO4_IOCTL_AO_RANGE");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


