// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/samples/sbtest/main.h $
// $Rev: 53428 $
// $Date: 2023-07-20 13:03:43 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Sample Application: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include <errno.h>
#include <semaphore.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "16hsdi4ao4_main.h"



// macros *********************************************************************

#define	SIZEOF_ARRAY(a)			(sizeof((a)) / sizeof((a)[0]))



// data types *****************************************************************

typedef struct
{
	// Application Settings

	s32	continuous;		// > 0 = ignore errors, < 0 = stop on errors
	s32	index;			// device index
	s32	minute_limit;	// for continuous testing
	int	qty;			// Number of devices detected.
	s32	test_limit;		// for continuous testing

	int	fd;				// File descriptor for the device to access.

	// Device Settings

} args_t;

typedef enum
{
	SERVICE_END_LIST,	// Ends list of service_data_t structures.
	SERVICE_NONE,		// Do nothing.
	SERVICE_NORMAL,
	SERVICE_REG_MOD,
	SERVICE_REG_READ,
	SERVICE_REG_SHOW,
	SERVICE_REG_TEST,
	SERVICE_REG_WRITE,	// arg = # of times to write
	SERVICE_SLEEP,		// arg = # of seconds to sleep
	SERVICE_SLEEP_MS,	// arg = # of milliseconds to sleep
	SERVICE_IOCTL_GET,
	SERVICE_IOCTL_SET
} service_t;

typedef struct
{
	service_t		service;
	unsigned long	cmd;	// IOCTL code
	s32				arg;	// The IOCTL data argument.
	u32				reg;	// The register to access. Use -1 to ignore.
	u32				mask;	// The register bits of interest.
	u32				value;	// The value expected for the bits of interest.
} service_data_t;



// prototypes *****************************************************************

int	ai_buf_clear_test(int fd);			// HSDI4AO4_IOCTL_AI_BUF_CLEAR
int	ai_buf_level_test(int fd);			// HSDI4AO4_IOCTL_AI_BUF_LEVEL
int	ai_buf_overflow_test(int fd);		// HSDI4AO4_IOCTL_AI_BUF_OVERFLOW
int	ai_buf_thr_lvl_test(int fd);		// HSDI4AO4_IOCTL_AI_BUF_THR_LVL
int	ai_buf_thr_sts_test(int fd);		// HSDI4AO4_IOCTL_AI_BUF_THR_STS
int	ai_buf_underflow_test(int fd);		// HSDI4AO4_IOCTL_AI_BUF_UNDERFLOW
int	ai_burst_enable_test(int fd);		// HSDI4AO4_IOCTL_AI_BURST_ENABLE
int	ai_burst_size_test(int fd);			// HSDI4AO4_IOCTL_AI_BURST_SIZE
int	ai_burst_status_test(int fd);		// HSDI4AO4_IOCTL_AI_BURST_STATUS
int	ai_chan_sel_test(int fd);			// HSDI4AO4_IOCTL_AI_CHAN_SEL
int	ai_clock_mode_test(int fd);			// HSDI4AO4_IOCTL_AI_CLOCK_MODE
int	ai_enable_test(int fd);				// HSDI4AO4_IOCTL_AI_ENABLE
int	ai_mode_test(int fd);				// HSDI4AO4_IOCTL_AI_MODE
int	ai_range_test(int fd);				// HSDI4AO4_IOCTL_AI_RANGE
int	ai_sw_clock_test(int fd);			// HSDI4AO4_IOCTL_AI_SW_CLOCK
int	ai_sw_trigger_test(int fd);			// HSDI4AO4_IOCTL_AI_SW_TRIGGER
int	ao_buf_clear_test(int fd);			// HSDI4AO4_IOCTL_AO_BUF_CLEAR
int	ao_buf_empty_test(int fd);			// HSDI4AO4_IOCTL_AO_BUF_EMPTY
int	ao_buf_level_test(int fd);			// HSDI4AO4_IOCTL_AO_BUF_LEVEL
int	ao_buf_load_req_test(int fd);		// HSDI4AO4_IOCTL_AO_BUF_LOAD_REQ
int	ao_buf_load_sts_test(int fd);		// HSDI4AO4_IOCTL_AO_BUF_LOAD_STS
int	ao_buf_mode_test(int fd);			// HSDI4AO4_IOCTL_AO_BUF_MODE
int	ao_buf_over_data_test(int fd);		// HSDI4AO4_IOCTL_AO_BUF_OVER_DATA
int	ao_buf_over_fram_test(int fd);		// HSDI4AO4_IOCTL_AO_BUF_OVER_FRAM
int	ao_buf_thr_lvl_test(int fd);		// HSDI4AO4_IOCTL_AO_BUF_THR_LVL
int	ao_buf_thr_sts_test(int fd);		// HSDI4AO4_IOCTL_AO_BUF_THR_STS
int	ao_burst_enable_test(int fd);		// HSDI4AO4_IOCTL_AO_BURST_ENABLE
int	ao_burst_status_test(int fd);		// HSDI4AO4_IOCTL_AO_BURST_STATUS
int	ao_chan_sel_test(int fd);			// HSDI4AO4_IOCTL_AO_CHAN_SEL
int	ao_clock_mode_test(int fd);			// HSDI4AO4_IOCTL_AO_CLOCK_MODE
int	ao_clock_status_test(int fd);		// HSDI4AO4_IOCTL_AO_CLOCK_STATUS
int	ao_enable_test(int fd);				// HSDI4AO4_IOCTL_AO_ENABLE
int	ao_mode_test(int fd);				// HSDI4AO4_IOCTL_AO_MODE
int	ao_output_mode_test(int fd);		// HSDI4AO4_IOCTL_AO_OUTPUT_MODE
int	ao_range_test(int fd);				// HSDI4AO4_IOCTL_AO_RANGE
int	ao_sw_clock_test(int fd);			// HSDI4AO4_IOCTL_AO_SW_CLOCK
int	ao_sw_trigger_test(int fd);			// HSDI4AO4_IOCTL_AO_SW_TRIGGER
int	ao_sync_mode_test(int fd);			// HSDI4AO4_IOCTL_AO_SYNC_MODE
int	autocal_status_test(int fd);		// HSDI4AO4_IOCTL_AUTOCAL_STATUS
int	autocal_test(int fd);				// HSDI4AO4_IOCTL_AUTOCAL

int	data_format_test(int fd);			// HSDI4AO4_IOCTL_DATA_FORMAT
int	dio_dir_out_test(int fd);			// HSDI4AO4_IOCTL_DIO_DIR_OUT
int	dio_read_test(int fd);				// HSDI4AO4_IOCTL_DIO_READ
int	dio_write_test(int fd);				// HSDI4AO4_IOCTL_DIO_WRITE

int	gen_a_enable_test(int fd);			// HSDI4AO4_IOCTL_GEN_A_ENABLE
int	gen_a_ndiv_test(int fd);			// HSDI4AO4_IOCTL_GEN_A_NDIV
int	gen_a_nref_test(int fd);			// HSDI4AO4_IOCTL_GEN_A_NREF
int	gen_a_nvco_test(int fd);			// HSDI4AO4_IOCTL_GEN_A_NVCO
int	gen_b_enable_test(int fd);			// HSDI4AO4_IOCTL_GEN_B_ENABLE
int	gen_b_ndiv_test(int fd);			// HSDI4AO4_IOCTL_GEN_B_NDIV
int	gen_c_enable_test(int fd);			// HSDI4AO4_IOCTL_GEN_C_ENABLE
int	gen_c_ndiv_test(int fd);			// HSDI4AO4_IOCTL_GEN_C_NDIV

int	initialize_test(int fd);			// HSDI4AO4_IOCTL_INITIALIZE
int	irq_enable_test(int fd);			// HSDI4AO4_IOCTL_IRQ_ENABLE

int	master_clock_adj_test(int fd);		// HSDI4AO4_IOCTL_MASTER_CLOCK_ADJ
int	master_clock_test(int fd);			// HSDI4AO4_IOCTL_MASTER_CLOCK

int	perform_tests(const args_t* args);

int	query_test(int fd);					// HSDI4AO4_IOCTL_QUERY

int	reg_mod_test(int fd);				// HSDI4AO4_IOCTL_REG_MOD
int	reg_read_test(int fd);				// HSDI4AO4_IOCTL_REG_READ
int	reg_write_test(int fd);				// HSDI4AO4_IOCTL_REG_WRITE
int	rx_io_abort_test(int fd);			// HSDI4AO4_IOCTL_RX_IO_ABORT
int	rx_io_mode_test(int fd);			// HSDI4AO4_IOCTL_RX_IO_MODE
int	rx_io_overflow_test(int fd);		// HSDI4AO4_IOCTL_RX_IO_OVERFLOW
int	rx_io_timeout_test(int fd);			// HSDI4AO4_IOCTL_RX_IO_TIMEOUT
int	rx_io_underflow_test(int fd);		// HSDI4AO4_IOCTL_RX_IO_UNDERFLOW

int	service_ioctl_reg_get_list	(int fd, int line, const service_data_t* list);
int	service_ioctl_set_list		(int fd, int line, const service_data_t* list);
int	service_ioctl_set_get_list	(int fd, int line, const service_data_t* list);
int	service_ioctl_set_reg_list	(int fd, int line, const service_data_t* list);

int	trigger_mode_test(int fd);			// HSDI4AO4_IOCTL_TRIGGER_MODE
int	tx_io_abort_test(int fd);			// HSDI4AO4_IOCTL_TX_IO_ABORT
int	tx_io_mode_test(int fd);			// HSDI4AO4_IOCTL_TX_IO_MODE
int	tx_io_overflow_test(int fd);		// HSDI4AO4_IOCTL_TX_IO_OVERFLOW
int	tx_io_timeout_test(int fd);			// HSDI4AO4_IOCTL_TX_IO_TIMEOUT

int	wait_cancel_test(int fd);			// HSDI4AO4_IOCTL_WAIT_CANCEL
int	wait_event_test(int fd);			// HSDI4AO4_IOCTL_WAIT_EVENT
int	wait_status_test(int fd);			// HSDI4AO4_IOCTL_WAIT_STATUS



#endif
