// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/samples/sbtest/reg_mod.c $
// $Rev: 52977 $
// $Date: 2023-05-02 14:11:22 -0500 (Tue, 02 May 2023) $

// 16HSDI4AO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const struct
	{
		u32	reg;
		u32	mask;
	} list[]	=
	{
		// reg					mask
		{ HSDI4AO4_GSC_ICR,		0x00FFFFFF	},
		{ HSDI4AO4_GSC_IBTR,	0x0007FFFF	},
		{ HSDI4AO4_GSC_OBTR,	0x0007FFFF	}
	};

	u32			bit;
	int			errs	= 0;
	int			i;
	gsc_reg_t	parm;
	int			ret;
	u32			value;

	errs	+= hsdi4ao4_initialize(fd, -1, 0);

	for (i = 0; (errs == 0) && (i < SIZEOF_ARRAY(list)); i++)
	{

		for (bit = 0x1; bit; bit <<= 1)
		{
			if ((list[i].mask & bit) == 0)
				continue;

			// Read the register value so we can restore it later.
			parm.reg	= list[i].reg;
			ret			= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_REG_READ, (void*) &parm);
			value		= parm.value;

			if (ret)
			{
				errs	= 1;
				printf("FAIL <---  (%d. hsdi4ao4_ioctl() failure, error %d)\n", __LINE__, ret);
				break;
			}

			// Set the bit low.
			parm.value	= 0;
			parm.mask	= bit;
			ret			= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_REG_MOD, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf("FAIL <---  (%d. hsdi4ao4_ioctl() failure, error %d)\n", __LINE__, ret);
				break;
			}

			// Now verify that the bit is low.
			ret	= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_REG_READ, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf("FAIL <---  (%d. hsdi4ao4_ioctl() failure, error %d)\n", __LINE__, ret);
				break;
			}

			if (parm.value & bit)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, bit not low, mask 0x%lX, got 0x%lX)\n",
						__LINE__,
						i,
						(long) bit,
						(long) parm.value);
				break;
			}

			// Now restore the register to its previous value.
			parm.value	= value;
			ret			= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_REG_WRITE, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf("FAIL <---  (%d. hsdi4ao4_ioctl() failure, error %d)\n", __LINE__, ret);
				break;
			}

			// Set the bit high.
			parm.value	= bit;
			parm.mask	= bit;
			ret			= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_REG_MOD, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf("FAIL <---  (%d. hsdi4ao4_ioctl() failure, error %d)\n", __LINE__, ret);
				break;
			}

			// Now verify that the bit is high.
			ret	= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_REG_READ, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf("FAIL <---  (%d. hsdi4ao4_ioctl() failure, error %d)\n", __LINE__, ret);
				break;
			}

			if ((parm.value & bit) == 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, bit not high, mask 0x%lX, got 0x%lX)\n",
						__LINE__,
						i,
						(long) bit,
						(long) parm.value);
				break;
			}

			// Now restore the register to its previous value.
			parm.value	= value;
			ret			= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_REG_WRITE, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf("FAIL <---  (%d. hsdi4ao4_ioctl() failure, error %d)\n", __LINE__, ret);
				break;
			}
		}
	}

	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	reg_mod_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSDI4AO4_IOCTL_REG_MOD.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int reg_mod_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSDI4AO4_IOCTL_REG_MOD");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


