// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/samples/sbtest/reg_read.c $
// $Rev: 52977 $
// $Date: 2023-05-02 14:11:22 -0500 (Tue, 02 May 2023) $

// 16HSDI4AO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const struct
	{
		u32	reg;
		u32	value;
	} list[]	=
	{
		// reg					value
		{ HSDI4AO4_GSC_BCR,		0		},
		{ HSDI4AO4_GSC_DIOPR,	0		},
		{ HSDI4AO4_GSC_AOC0R,	0		},
		{ HSDI4AO4_GSC_AOC1R,	0		},
		{ HSDI4AO4_GSC_AOC2R,	0		},
		{ HSDI4AO4_GSC_AOC3R,	0		},
		{ HSDI4AO4_GSC_AIBR,	0		},
		{ HSDI4AO4_GSC_RGAR,	0		},
		{ HSDI4AO4_GSC_RGBR,	0		},
		{ HSDI4AO4_GSC_ICR,		0		},
		{ HSDI4AO4_GSC_IBSR,	0		},
		{ HSDI4AO4_GSC_IBTR,	0		},
		{ HSDI4AO4_GSC_PSR,		0		},
		{ HSDI4AO4_GSC_ACR,		0		},
		{ HSDI4AO4_GSC_AVR,		0		},
		{ HSDI4AO4_GSC_BOOR,	0		},
		{ HSDI4AO4_GSC_OBTR,	0		},
		{ HSDI4AO4_GSC_OBSR,	0		},
		{ HSDI4AO4_GSC_AOBR,	0		},
		{ HSDI4AO4_GSC_RGCR,	0		},
		{ HSDI4AO4_GSC_MCAR,	0		},

		{ GSC_PCI_9056_VIDR,	0x10B5	},
		{ GSC_PCI_9056_DIDR,	0x9056	},
		{ GSC_PCI_9056_SVID,	0x10B5	},
		{ GSC_PCI_9056_SID,		0x3428	},

		{ GSC_PLX_9056_VIDR,	0x10B5	},
		{ GSC_PLX_9056_DIDR,	0x9056	},
	};

	int			errs	= 0;
	int			i;
	gsc_reg_t	parm;
	int			ret;

	errs	+= hsdi4ao4_initialize(fd, -1, 0);

	for (i = 0; i < SIZEOF_ARRAY(list); i++)
	{
		parm.reg	= list[i].reg;
		parm.value	= 0xDEADBEEF;
		parm.mask	= 0xBEEFDEAD;
		ret			= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_REG_READ, (void*) &parm);

		if (ret)
		{
			errs	= 1;
			printf("FAIL <---  (%d. hsdi4ao4_ioctl() failure, error %d)\n", __LINE__, ret);
			break;
		}

		if (parm.value == 0xDEADBEEF)
		{
			errs	= 1;
			printf("FAIL <---  (%d. i %d, value not changed)\n", __LINE__, i);
			break;
		}

		if (parm.mask != 0xBEEFDEAD)
		{
			errs	= 1;
			printf("FAIL <---  (%d. i %d, mask changed)\n", __LINE__, i);
			break;
		}

		if ((list[i].value) && (parm.value != list[i].value))
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, got 0x%lX, expected 0x%lX)\n",
					__LINE__,
					i,
					(long) parm.value,
					(long) list[i].value);
			break;
		}
	}

	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	reg_read_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSDI4AO4_IOCTL_REG_READ.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int reg_read_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSDI4AO4_IOCTL_REG_READ");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


