// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/samples/sbtest/reg_write.c $
// $Rev: 52977 $
// $Date: 2023-05-02 14:11:22 -0500 (Tue, 02 May 2023) $

// 16HSDI4AO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const struct
	{
		u32	reg;
		u32	value;
		u32	mask;
	} list[]	=
	{
		// reg					value		mask
		{ HSDI4AO4_GSC_ICR,		0x00AA5555,	0x00FFFFFF	},
		{ HSDI4AO4_GSC_ICR,		0x00000000,	0x00FFFFFF	},

		{ HSDI4AO4_GSC_IBTR,	0x0005AAAA,	0x0007FFFF	},
		{ HSDI4AO4_GSC_IBTR,	0x0003FFFE,	0x0007FFFF	},

		{ HSDI4AO4_GSC_OBTR,	0x0005AAAA,	0x0007FFFF	},
		{ HSDI4AO4_GSC_OBTR,	0x0003FFFE,	0x0007FFFF	},
	};

	int			errs	= 0;
	int			i;
	gsc_reg_t	parm;
	int			ret;

	errs	+= hsdi4ao4_initialize(fd, -1, 0);

	for (i = 0; i < SIZEOF_ARRAY(list); i++)
	{
		parm.reg	= list[i].reg;
		parm.value	= list[i].value;
		parm.mask	= 0xBEEFDEAD;
		ret			= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_REG_WRITE, (void*) &parm);

		if (ret)
		{
			errs	= 1;
			printf("FAIL <---  (%d. hsdi4ao4_ioctl() failure, error %d)\n", __LINE__, ret);
			break;
		}

		if (parm.mask != 0xBEEFDEAD)
		{
			errs	= 1;
			printf("FAIL <---  (%d. i %d, mask changed)\n", __LINE__, i);
			break;
		}

		// Now read the register and verify the bits of interest.
		ret	= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_REG_READ, (void*) &parm);

		if (ret)
		{
			errs	= 1;
			printf("FAIL <---  (%d. hsdi4ao4_ioctl() failure, error %d)\n", __LINE__, ret);
			break;
		}

		if ((list[i].value & list[i].mask) != (parm.value & list[i].mask))
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, mask 0x%lX, got 0x%lX, expected 0x%lX)\n",
					__LINE__,
					i,
					(long) list[i].mask,
					(long) parm.value,
					(long) list[i].value);
			break;
		}
	}

	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	reg_write_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSDI4AO4_IOCTL_REG_WRITE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int reg_write_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSDI4AO4_IOCTL_REG_WRITE");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


