// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/samples/sbtest/tx_io_abort.c $
// $Rev: 53428 $
// $Date: 2023-07-20 13:03:43 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	sem_t	sem_enter;
static	sem_t	sem_exit;



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no persistent bits to test with this service.
	return(0);
}



//*****************************************************************************
static int _tx_fifo_fill(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_MODE,
			/* arg		*/	HSDI4AO4_AO_MODE_FIFO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	1024,
			/* reg		*/	HSDI4AO4_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	boor;
	int	errs	= 0;
	int	i;

	for (i = 0; i < 1024; i++)
	{
		errs	+= service_ioctl_set_list(fd, __LINE__, list);
		errs	+= hsdi4ao4_reg_read(fd, -1, 0, HSDI4AO4_GSC_BOOR, &boor);

		if (boor & 0x8000)
		{
			// The Tx FIFO is FULL.
			break;
		}
		else
		{
			// The Output Buffer is not FULL
			continue;
		}
	}

	return(errs);
}



//*****************************************************************************
static int _abort_none_test(int fd)
{
	int	errs	= 0;
	s32	get;
	int	ret;

	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	ret		= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_TX_IO_ABORT, &get);
	errs	+= ret ? 1 : 0;

	if ((errs == 0) && (get))
	{
		errs++;
		printf(	"FAIL <---  (%d. got %ld, expected %ld)\n",
				__LINE__,
				(long) get,
				(long) 0);
	}

	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _tx_thread(void* arg)
{
	char	buffer[1024]	= { 0 };
	int		fd				= (int) (long) arg;

	sem_post(&sem_enter);
	hsdi4ao4_write(fd, buffer, sizeof(buffer));
	sem_post(&sem_exit);

	return(0);
}



//*****************************************************************************
static int _abort_one_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_TX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_TX_IO_OVERFLOW,
			/* arg		*/	HSDI4AO4_IO_OVERFLOW_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_TX_IO_TIMEOUT,
			/* arg		*/	15,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int			errs	= 0;
	s32			get;
	int			i;
	int			ret;
	os_thread_t	thread;

	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	errs	+= _tx_fifo_fill(fd);
	errs	+= service_ioctl_set_list(fd, __LINE__, list);

	sem_init(&sem_enter, 0, 0);
	sem_init(&sem_exit, 0, 0);
	memset(&thread, 0, sizeof(thread));
	errs	+= os_thread_create(
				&thread,
				"Tx I/O Abort",
				_tx_thread,
				(void*) (long) fd);

	// Wait for the thread to execute.
	sem_wait(&sem_enter);

	if (errs == 0)
	{
		for (i = 0; i < 20; i++)
		{
			ret		= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_TX_IO_ABORT, &get);
			errs	+= ret ? 1 : 0;

			if ((errs) || (get))
				break;

			os_sleep_ms(1000);
		}

		if (errs)
		{
		}
		else if (get == 0)
		{
			errs++;
			printf("FAIL <---  (%d. abort not detected)\n", __LINE__);
		}
	}

	// Wait for the thread to finish.
	sem_wait(&sem_exit);

	// Finish up.
	errs	+= os_thread_destroy(&thread);
	errs	+= hsdi4ao4_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	int errs	= 0;

	errs	+= _abort_none_test(fd);
	errs	+= _abort_one_test(fd);
	return(errs);
}



/******************************************************************************
*
*	Function:	tx_io_abort_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSDI4AO4_IOCTL_TX_IO_ABORT.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int tx_io_abort_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSDI4AO4_IOCTL_TX_IO_ABORT");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


