// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/samples/sbtest/wait_event.c $
// $Rev: 53428 $
// $Date: 2023-07-20 13:03:43 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1_SEC				(1000L)
#define	_15_SEC				(15L * 1000L)
#define	_30_SEC				(30L * 1000L)



// variables ******************************************************************

static	int			io_stat;
static	sem_t		sem_enter;
static	sem_t		sem_exit;
static	gsc_wait_t	wait_dst;
static	int			wait_fd;



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no settings bits to test here.
	return(0);
}



//*****************************************************************************
static int _wait_thread(void* arg)
{
	int	ret;

	sem_post(&sem_enter);
	ret	= hsdi4ao4_ioctl(wait_fd, HSDI4AO4_IOCTL_WAIT_EVENT, &wait_dst);

	if (ret)
		wait_dst.flags	= 0xFFFFFFFF;

	sem_post(&sem_exit);
	return(0);
}



//*****************************************************************************
static int	_wait_main_dma0(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Select DMA mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_BMDMA,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_RX_IO_OVERFLOW,
			/* arg		*/	HSDI4AO4_IO_OVERFLOW_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_RX_IO_UNDERFLOW,
			/* arg		*/	HSDI4AO4_IO_UNDERFLOW_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_CLOCK_MODE,
			/* arg		*/	HSDI4AO4_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_GEN_A_ENABLE,
			/* arg		*/	HSDI4AO4_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Wait for some data.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	buf[1024];
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= hsdi4ao4_initialize(fd, -1, 0);
		errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);

		// Now perform a DMA read to trigger the wait.

		if (errs == 0)
		{
			ret	= hsdi4ao4_read(fd, buf, sizeof(buf));

			if (ret != sizeof(buf))
			{
				errs++;
				printf(	"FAIL <---  (%d. hsdi4ao4_read(): got %ld, expected %ld)\n",
						__LINE__,
						(long) ret,
						(long) sizeof(buf));
			}
		}

		ret	= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_main_dma1(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Select DMA mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_TX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_DMDMA,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_TX_IO_OVERFLOW,
			/* arg		*/	HSDI4AO4_IO_OVERFLOW_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	buf[1024]	= { 0 };
	int	errs		= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= hsdi4ao4_initialize(fd, -1, 0);
		errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);

		// Now perform a DMA write to trigger the wait.

		if (errs == 0)
		{
			ret	= hsdi4ao4_write(fd, buf, sizeof(buf));

			if ((ret >= 0) && (ret != sizeof(buf)))
			{
				errs++;
				printf(	"FAIL <---  (%d. hsdi4ao4_write(): got %ld, expected %ld)\n",
						__LINE__,
						(long) ret,
						(long) sizeof(buf));
			}
		}

		ret	= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_main_gsc(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AUTOCAL,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, __LINE__, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_autocal_done(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AUTOCAL,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, __LINE__, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_ai_thresh_h2l(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_IRQ_ENABLE,
			/* arg		*/	HSDI4AO4_IRQ_AI_THRESH_H2L,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BUF_THR_LVL,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_CLOCK_MODE,
			/* arg		*/	HSDI4AO4_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_GEN_A_ENABLE,
			/* arg		*/	HSDI4AO4_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Wait for some data.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, __LINE__, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_ai_thresh_l2h(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_IRQ_ENABLE,
			/* arg		*/	HSDI4AO4_IRQ_AI_THRESH_L2H,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BUF_THR_LVL,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_CLOCK_MODE,
			/* arg		*/	HSDI4AO4_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_GEN_A_ENABLE,
			/* arg		*/	HSDI4AO4_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Wait for some data.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	50,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, __LINE__, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_ai_fault(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_IRQ_ENABLE,
			/* arg		*/	HSDI4AO4_IRQ_AI_FAULT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_READ,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	HSDI4AO4_GSC_AIBR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, __LINE__, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_ai_burst_start(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_IRQ_ENABLE,
			/* arg		*/	HSDI4AO4_IRQ_AI_BURST_START,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BURST_ENABLE,
			/* arg		*/	HSDI4AO4_BURST_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BURST_SIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_CLOCK_MODE,
			/* arg		*/	HSDI4AO4_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_GEN_A_ENABLE,
			/* arg		*/	HSDI4AO4_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_SW_TRIGGER,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, __LINE__, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_ai_burst_done(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_IRQ_ENABLE,
			/* arg		*/	HSDI4AO4_IRQ_AI_BURST_DONE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BURST_ENABLE,
			/* arg		*/	HSDI4AO4_BURST_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_BURST_SIZE,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_CLOCK_MODE,
			/* arg		*/	HSDI4AO4_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_GEN_A_ENABLE,
			/* arg		*/	HSDI4AO4_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_SW_TRIGGER,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, __LINE__, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_ai_clock(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_IRQ_ENABLE,
			/* arg		*/	HSDI4AO4_IRQ_AI_CLOCK,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_CLOCK_MODE,
			/* arg		*/	HSDI4AO4_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_GEN_A_ENABLE,
			/* arg		*/	HSDI4AO4_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, __LINE__, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_ao_clock(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_IRQ_ENABLE,
			/* arg		*/	HSDI4AO4_IRQ_AO_CLOCK,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_CLOCK_MODE,
			/* arg		*/	HSDI4AO4_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_GEN_C_ENABLE,
			/* arg		*/	HSDI4AO4_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, __LINE__, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_dio_0_l2h(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_IRQ_ENABLE,
			/* arg		*/	HSDI4AO4_IRQ_DIO_0_L2H,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_DIO_DIR_OUT,
			/* arg		*/	HSDI4AO4_DIO_DIR_OUT_B0_OT_B1_OT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_DIO_WRITE,
			/* arg		*/	0x0000,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_DIO_WRITE,
			/* arg		*/	0xFFFF,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, __LINE__, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_ao_thresh_h2l(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_IRQ_ENABLE,
			/* arg		*/	HSDI4AO4_IRQ_AO_THRESH_H2L,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_BUF_THR_LVL,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_MODE,
			/* arg		*/	HSDI4AO4_AO_MODE_FIFO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	HSDI4AO4_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, __LINE__, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_ao_thresh_l2h(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_IRQ_ENABLE,
			/* arg		*/	HSDI4AO4_IRQ_AO_THRESH_L2H,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_BUF_THR_LVL,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_MODE,
			/* arg		*/	HSDI4AO4_AO_MODE_FIFO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	HSDI4AO4_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, __LINE__, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_ao_load_ready_h2l(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_IRQ_ENABLE,
			/* arg		*/	HSDI4AO4_IRQ_AO_LOAD_RDY_H2L,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_MODE,
			/* arg		*/	HSDI4AO4_AO_MODE_FIFO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_BUF_MODE,
			/* arg		*/	HSDI4AO4_BUF_MODE_CIRC,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, __LINE__, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_ao_load_ready_l2h(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_IRQ_ENABLE,
			/* arg		*/	HSDI4AO4_IRQ_AO_LOAD_RDY_L2H,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_MODE,
			/* arg		*/	HSDI4AO4_AO_MODE_FIFO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, __LINE__, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_ao_burst_ready(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_IRQ_ENABLE,
			/* arg		*/	HSDI4AO4_IRQ_AO_BURST_READY,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_MODE,
			/* arg		*/	HSDI4AO4_AO_MODE_FIFO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_BURST_ENABLE,
			/* arg		*/	HSDI4AO4_BURST_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_CLOCK_MODE,
			/* arg		*/	HSDI4AO4_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_GEN_C_ENABLE,
			/* arg		*/	HSDI4AO4_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	10,
			/* reg		*/	HSDI4AO4_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, __LINE__, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_ao_fault(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_IRQ_ENABLE,
			/* arg		*/	HSDI4AO4_IRQ_AO_FAULT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_MODE,
			/* arg		*/	HSDI4AO4_AO_MODE_FIFO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	257L * 1024,
			/* reg		*/	HSDI4AO4_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, __LINE__, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_io_rx_done(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_RX_IO_OVERFLOW,
			/* arg		*/	HSDI4AO4_IO_OVERFLOW_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_RX_IO_UNDERFLOW,
			/* arg		*/	HSDI4AO4_IO_UNDERFLOW_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_CLOCK_MODE,
			/* arg		*/	HSDI4AO4_CLOCK_MODE_OUT,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_GEN_A_ENABLE,
			/* arg		*/	HSDI4AO4_GEN_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AI_ENABLE,
			/* arg		*/	HSDI4AO4_AIO_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Wait for some data.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	buf[1024];
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= hsdi4ao4_initialize(fd, -1, 0);
		errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);

		// Now perform a DMA read to trigger the wait.

		if (errs == 0)
		{
			ret	= hsdi4ao4_read(fd, buf, sizeof(buf));

			if (ret != sizeof(buf))
			{
				errs++;
				printf(	"FAIL <---  (%d. hsdi4ao4_read(): got %ld, expected %ld)\n",
						__LINE__,
						(long) ret,
						(long) sizeof(buf));
			}
		}

		ret	= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_io_rx_error(int fd, int supported)
{
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		// Now perform a read to trigger the wait.
		ret	= hsdi4ao4_read(fd, NULL, 3);

		if (ret >= 0)
		{
			errs++;
			printf("FAIL <---  (%d. expected %d, got %d)\n", __LINE__, -1, ret);
		}

		ret	= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_io_rx_timeout(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Select PIO mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_RX_IO_OVERFLOW,
			/* arg		*/	HSDI4AO4_IO_OVERFLOW_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_RX_IO_UNDERFLOW,
			/* arg		*/	HSDI4AO4_IO_UNDERFLOW_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	buf[1024];
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= hsdi4ao4_initialize(fd, -1, 0);

		// Verify that the status is an overflow.
		errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);

		// Now perform a read to trigger the wait.

		if (errs == 0)
		{
			ret	= hsdi4ao4_read(fd, buf, SIZEOF_ARRAY(buf));

			if (ret)
			{
				errs++;
				printf(	"FAIL <---  (%d. hsdi4ao4_read(): got %ld, expected %ld)\n",
						__LINE__,
						(long) ret,
						(long) 0);
			}
		}

		errs	+= hsdi4ao4_initialize(fd, -1, 0);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int _read_thread(void* arg)
{
	u32		buf[1024];
	int		fd	= (int) (long) arg;
	long	ret;

	ret	= hsdi4ao4_read(fd, buf, sizeof(buf));

	if (ret == 0)
	{
		io_stat	= 1;
	}
	else
	{
		io_stat	= 2;
		printf(	"FAIL <---  (%d. hsdi4ao4_read() returned %ld, expected 0)\n",
				__LINE__,
				ret);
	}

	return(0);
}



//*****************************************************************************
static int	_wait_io_rx_abort(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{					// Select PIO mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_RX_IO_OVERFLOW,
			/* arg		*/	HSDI4AO4_IO_OVERFLOW_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_RX_IO_UNDERFLOW,
			/* arg		*/	HSDI4AO4_IO_UNDERFLOW_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int			errs	= 0;
	int			i;
	int			ret;
	s32			set;
	os_thread_t	thread;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= hsdi4ao4_initialize(fd, -1, 0);

		// Verify that the status is an overflow.
		errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);

		// Now start the read thread so we can do the abort.
		memset(&thread, 0, sizeof(thread));

		if (errs == 0)
		{
			errs	+= os_thread_create(	&thread,
											"Rx",
											_read_thread,
											(void*) (long) fd);
		}

		for (i = 0; (errs == 0) && (i < 100); i++)
		{
			set		= 0;
			ret		= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_RX_IO_ABORT, &set);
			errs	+= ret ? 1 : 0;

			if (errs)
				break;

			if (set)
				break;

			os_sleep_ms(100);
		}

		// Wait for the thread to report its status.

		for (i = 0; i < 100; i++)
		{
			if (errs)
				break;

			if (io_stat == 0)
			{
				os_sleep_ms(100);
				continue;
			}

			if (io_stat == 1)
				break;

			errs++;
			break;
		}

		errs	+= os_thread_destroy(&thread);
		errs	+= hsdi4ao4_initialize(fd, -1, 0);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_io_tx_done(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_TX_IO_OVERFLOW,
			/* arg		*/	HSDI4AO4_IO_OVERFLOW_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	buf[1024]	= { 0 };
	int	errs		= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= hsdi4ao4_initialize(fd, -1, 0);
		errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);

		// Now perform a DMA read to trigger the wait.

		if (errs == 0)
		{
			ret	= hsdi4ao4_write(fd, buf, sizeof(buf));

			if ((ret >= 0) && (ret != sizeof(buf)))
			{
				errs++;
				printf(	"FAIL <---  (%d. hsdi4ao4_write(): got %ld, expected %ld)\n",
						__LINE__,
						(long) ret,
						(long) SIZEOF_ARRAY(buf));
			}
		}

		ret	= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_io_tx_error(int fd, int supported)
{
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		// Now perform a write to trigger the wait.
		ret	= hsdi4ao4_write(fd, NULL, 3);

		if (ret >= 0)
		{
			errs++;
			printf("FAIL <---  (%d. expected %d, got %d)\n", __LINE__, -1, ret);
		}

		ret	= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_io_tx_timeout(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{					// Select PIO mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_TX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_MODE,
			/* arg		*/	HSDI4AO4_AO_MODE_FIFO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_TX_IO_OVERFLOW,
			/* arg		*/	HSDI4AO4_IO_OVERFLOW_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_TX_IO_TIMEOUT,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	257L * 1024,
			/* reg		*/	HSDI4AO4_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	buf[1024]	= { 0 };
	int	errs		= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= hsdi4ao4_initialize(fd, -1, 0);
		errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);

		// Now perform a read to trigger the wait.

		if (errs == 0)
		{
			ret	= hsdi4ao4_write(fd, buf, SIZEOF_ARRAY(buf));

			if (ret != 0)
			{
				errs++;
				printf(	"FAIL <---  (%d. hsdi4ao4_write(): got %ld, expected %ld)\n",
						__LINE__,
						(long) ret,
						(long) 0);
			}
		}

		errs	+= hsdi4ao4_initialize(fd, -1, 0);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int _write_thread(void* arg)
{
	u32		buf[1024];
	int		fd			= (int) (long) arg;
	long	ret;

	memset(buf, 0, sizeof(buf));
	ret	= hsdi4ao4_write(fd, buf, sizeof(buf));

	if (ret == 0)
	{
		io_stat	= 1;
	}
	else
	{
		io_stat	= 2;
		printf("FAIL <---  (%d. hsdi4ao4_write() returned %ld, expected 0)\n",
				__LINE__,
				ret);
	}

	return(0);
}



//*****************************************************************************
static int	_wait_io_tx_abort(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{					// Select PIO mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_TX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_TX_IO_OVERFLOW,
			/* arg		*/	HSDI4AO4_IO_OVERFLOW_IGNORE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_TX_IO_TIMEOUT,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	HSDI4AO4_IOCTL_AO_MODE,
			/* arg		*/	HSDI4AO4_AO_MODE_FIFO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_REG_WRITE,
			/* cmd		*/	0,
			/* arg		*/	257L * 1024,
			/* reg		*/	HSDI4AO4_GSC_AOBR,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int			errs	= 0;
	int			i;
	int			ret;
	s32			set;
	os_thread_t	thread;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= hsdi4ao4_initialize(fd, -1, 0);

		// Verify that the status is an overflow.
		errs	+= service_ioctl_set_reg_list(fd, __LINE__, list);

		// Now start the write thread so we can do the abort.
		memset(&thread, 0, sizeof(thread));

		if (errs == 0)
		{
			errs	+= os_thread_create(	&thread,
											"Tx",
											_write_thread,
											(void*) (long) fd);
		}

		for (i = 0; (errs == 0) && (i < 100); i++)
		{
			set		= 0;
			ret		= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_TX_IO_ABORT, &set);
			errs	+= ret ? 1 : 0;

			if (errs)
				break;

			if (set)
				break;

			os_sleep_ms(100);
		}

		// Wait for the thread to report its status.

		for (i = 0; i < 100; i++)
		{
			if (errs)
				break;

			if (io_stat == 0)
			{
				os_sleep_ms(100);
				continue;
			}

			if (io_stat == 1)
				break;

			errs++;
			break;
		}

		errs	+= os_thread_destroy(&thread);
		errs	+= hsdi4ao4_initialize(fd, -1, 0);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_timeout(int fd, int supported)
{
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		os_sleep_ms(2000);
		ret	= 0;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_cancel(int fd, int supported)
{
	int			errs	= 0;
	int			ret;
	gsc_wait_t	wait;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		wait.flags			= 0;
		wait.main			= 0xFFFFFFFF;
		wait.gsc			= 0xFFFFFFFF;
		wait.alt			= 0xFFFFFFFF;
		wait.io				= 0xFFFFFFFF;
		wait.timeout_ms		= 0;
		wait.count			= 0;
		ret		= hsdi4ao4_ioctl(wait_fd, HSDI4AO4_IOCTL_WAIT_CANCEL, &wait);
		errs	+= ret ? 1 : 0;
		ret		= errs;
	}

	return(ret);
}



// variables ******************************************************************

static const struct
{
	const char*	name;
	int			(*function)(int fd, int supported);
	gsc_wait_t	wait;
} wait_src[]	=
{
	// name						function						flags			main								...			timeout_ms
	{ "DMA0",					_wait_main_dma0,				{ 0,			GSC_WAIT_MAIN_DMA0,					0, 0, 0,	_15_SEC	}	},
	{ "DMA1",					_wait_main_dma1,				{ 0,			GSC_WAIT_MAIN_DMA1,					0, 0, 0,	_15_SEC	}	},
	{ "GSC",					_wait_main_gsc,					{ 0,			GSC_WAIT_MAIN_GSC,					0, 0, 0,	_15_SEC	}	},

	// name						function						flags ...		gsc									...			timeout_ms
	{ "Autocal Done",			_wait_gsc_autocal_done,			{ 0, 0,			HSDI4AO4_WAIT_GSC_AUTOCAL_DONE,		0, 0,		_15_SEC	}	},
	{ "AI Threshold Hi->Low",	_wait_gsc_ai_thresh_h2l,		{ 0, 0,			HSDI4AO4_WAIT_GSC_AI_THRESH_H2L,	0, 0,		_15_SEC	}	},
	{ "AI Threshold Low->Hi",	_wait_gsc_ai_thresh_l2h,		{ 0, 0,			HSDI4AO4_WAIT_GSC_AI_THRESH_L2H,	0, 0,		_15_SEC	}	},
	{ "AI Fault",				_wait_gsc_ai_fault,				{ 0, 0,			HSDI4AO4_WAIT_GSC_AI_FAULT,			0, 0,		_15_SEC	}	},
	{ "AI Burst Start",			_wait_gsc_ai_burst_start,		{ 0, 0,			HSDI4AO4_WAIT_GSC_AI_BURST_START,	0, 0,		_15_SEC	}	},
	{ "AI Burst Done",			_wait_gsc_ai_burst_done,		{ 0, 0,			HSDI4AO4_WAIT_GSC_AI_BURST_DONE,	0, 0,		_15_SEC	}	},
	{ "AI Clock",				_wait_gsc_ai_clock,				{ 0, 0,			HSDI4AO4_WAIT_GSC_AI_CLOCK,			0, 0,		_15_SEC	}	},
	{ "AO Clock",				_wait_gsc_ao_clock,				{ 0, 0,			HSDI4AO4_WAIT_GSC_AO_CLOCK,			0, 0,		_15_SEC	}	},
	{ "DIO 0 Low->Hi",			_wait_gsc_dio_0_l2h,			{ 0, 0,			HSDI4AO4_WAIT_GSC_DIO_0_L2H,		0, 0,		_15_SEC	}	},
	{ "AO Threshold Hi->Low",	_wait_gsc_ao_thresh_h2l,		{ 0, 0,			HSDI4AO4_WAIT_GSC_AO_THRESH_H2L,	0, 0,		_15_SEC	}	},
	{ "AO Threshold Low->Hi",	_wait_gsc_ao_thresh_l2h,		{ 0, 0,			HSDI4AO4_WAIT_GSC_AO_THRESH_L2H,	0, 0,		_15_SEC	}	},
	{ "AO Load Ready Hi->Low",	_wait_gsc_ao_load_ready_h2l,	{ 0, 0,			HSDI4AO4_WAIT_GSC_AO_LOAD_RDY_HL,	0, 0,		_15_SEC	}	},
	{ "AO Load Ready Low->Hi",	_wait_gsc_ao_load_ready_l2h,	{ 0, 0,			HSDI4AO4_WAIT_GSC_AO_LOAD_RDY_LH,	0, 0,		_15_SEC	}	},
	{ "AO Burst Ready",			_wait_gsc_ao_burst_ready,		{ 0, 0,			HSDI4AO4_WAIT_GSC_AO_BURST_READY,	0, 0,		_15_SEC	}	},
	{ "AO Fault",				_wait_gsc_ao_fault,				{ 0, 0,			HSDI4AO4_WAIT_GSC_AO_FAULT,			0, 0,		_15_SEC	}	},

	// name						function						flags ...		io												timeout_ms
	{ "Rx Done",				_wait_io_rx_done,				{ 0, 0, 0, 0,	HSDI4AO4_WAIT_IO_RX_DONE,						_15_SEC	}	},
	{ "Rx Error",				_wait_io_rx_error,				{ 0, 0, 0, 0,	HSDI4AO4_WAIT_IO_RX_ERROR,						_15_SEC	}	},
	{ "Rx Timeout",				_wait_io_rx_timeout,			{ 0, 0, 0, 0,	HSDI4AO4_WAIT_IO_RX_TIMEOUT,					_15_SEC	}	},
	{ "Rx Abort",				_wait_io_rx_abort,				{ 0, 0, 0, 0,	HSDI4AO4_WAIT_IO_RX_ABORT,						_15_SEC	}	},
	{ "Tx Done",				_wait_io_tx_done,				{ 0, 0, 0, 0,	HSDI4AO4_WAIT_IO_TX_DONE,						_15_SEC	}	},
	{ "Tx Error",				_wait_io_tx_error,				{ 0, 0, 0, 0,	HSDI4AO4_WAIT_IO_TX_ERROR,						_15_SEC	}	},
	{ "Tx Timeout",				_wait_io_tx_timeout,			{ 0, 0, 0, 0,	HSDI4AO4_WAIT_IO_TX_TIMEOUT,					_15_SEC	}	},
	{ "Tx Abort",				_wait_io_tx_abort,				{ 0, 0, 0, 0,	HSDI4AO4_WAIT_IO_TX_ABORT,						_15_SEC	}	},

	// name						function						flags ...		io												timeout_ms
	{ "Wait Timeout",			_wait_timeout,					{ 0, 0, 0, 0,	HSDI4AO4_WAIT_IO_RX_DONE,						_1_SEC	}	},
	{ "Wait Cancel",			_wait_cancel,					{ 0, 0, 0, 0,	HSDI4AO4_WAIT_IO_RX_DONE,						_30_SEC	}	},

	// terminate list
	{ NULL, NULL, { 1 } }
};



//*****************************************************************************
static int _process_thread(int index)
{
	char		buf[64];
	int			errs	= 0;
	int			i;
	int			ret;
	os_thread_t	thread;
	gsc_wait_t	wait;

	for (;;)	// A convenience loop.
	{
		sem_init(&sem_enter, 0, 0);
		sem_init(&sem_exit, 0, 0);
		memset(&thread, 0, sizeof(thread));
		sprintf(buf, "wait event %d", index);
		wait_dst	= wait_src[index].wait;
		errs		+= os_thread_create(&thread, buf, _wait_thread, NULL);

		// Wait for the thread to become waiting.
		sem_wait(&sem_enter);

		if (errs == 0)
		{
			wait	= wait_src[index].wait;

			for (i = 0; i < 100; i++)
			{
				ret	= hsdi4ao4_ioctl(wait_fd, HSDI4AO4_IOCTL_WAIT_STATUS, &wait);

				if (ret)
				{
					printf(	"FAIL <---  (%d. status request %d: error %d)\n",
							__LINE__,
							index,
							ret);
					errs++;
					break;
				}

				if (wait.count == 0)
				{
					os_sleep_ms(100);
					continue;
				}

				if (wait.count == 1)
					break;

				errs++;
				printf(	"FAIL <---  (%d. invalid wait count %ld)\n",
						__LINE__,
						(long) wait.count);
				break;
			}
		}

		// Initiate the respective event.
		errs	+= wait_src[index].function(wait_fd, 0);

		if (errs)
			break;		// There was an error.

		// Verify that the wait ended as expected.

		if (wait_src[index].wait.flags == 0xFFFFFFFF)
			break;		// There was an error.

		// Wait for the resumed thread to run.
		sem_wait(&sem_exit);

		for (i = 0;; i++)
		{
			if (wait_dst.flags)
				break;

			if (i >= 100)
			{
				printf(	"FAIL <---  (%d. thread failed to resume)\n",
						__LINE__);
				errs++;
				break;
			}

			os_sleep_ms(100);
		}

		if (errs)
			break;		// There was an error.

		if (wait_src[index].wait.timeout_ms == _1_SEC)
		{
			if (wait_dst.flags != GSC_WAIT_FLAG_TIMEOUT)
			{
				printf(	"FAIL <---  (%d. flag: expect 0x%lX, got 0x%lX)\n",
						__LINE__,
						(long) GSC_WAIT_FLAG_TIMEOUT,
						(long) wait_dst.flags);
				errs++;
			}

			break;
		}

		if (wait_src[index].wait.timeout_ms == _15_SEC)
		{
			if (wait_dst.flags != GSC_WAIT_FLAG_DONE)
			{
				printf(	"FAIL <---  (%d. flag: expect 0x%lX, got 0x%lX)\n",
						__LINE__,
						(long) GSC_WAIT_FLAG_DONE,
						(long) wait_dst.flags);
				errs++;
			}

			break;
		}

		if (wait_src[index].wait.timeout_ms == _30_SEC)
		{
			if (wait_dst.flags != GSC_WAIT_FLAG_CANCEL)
			{
				printf(	"FAIL <---  (%d. flag: expect 0x%lX, got 0x%lX)\n",
						__LINE__,
						(long) GSC_WAIT_FLAG_TIMEOUT,
						(long) wait_dst.flags);
				errs++;
			}

			break;
		}

		printf("FAIL <---  (%d. INTERNAL ERROR)\n", __LINE__);
		errs++;
		break;
	}

	// Force termination, just in case of an error.
	wait.flags			= 0;
	wait.main			= 0xFFFFFFFF;
	wait.gsc			= 0xFFFFFFFF;
	wait.alt			= 0xFFFFFFFF;
	wait.io				= 0xFFFFFFFF;
	wait.timeout_ms		= 0;
	wait.count			= 0;
	ret		= hsdi4ao4_ioctl(wait_fd, HSDI4AO4_IOCTL_WAIT_CANCEL, &wait);
	errs	+= ret ? 1 : 0;
	errs	+= os_thread_destroy(&thread);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	int	errs	= 0;
	int	i;
	int	ret;

	printf("\n");
	gsc_label_level_inc();

	// Create threads awaiting each of the source crtiteria
	// and verify that it resumes when the criteria is met.

	for (i = 0; wait_src[i].wait.flags == 0; i++)
	{
		gsc_label(wait_src[i].name);
		ret	= wait_src[i].function(wait_fd, 1);

		if (ret > 0)
		{
		}
		else if (ret == 0)
		{
			printf("SKIPPED  (Not supported on this device.)\n");
		}
		else
		{
			wait_fd	= fd;
			memset(&wait_dst, 0, sizeof(wait_dst));
			errs	+= _process_thread(i);
		}
	}

	gsc_label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	wait_event_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service HSDI4AO4_IOCTL_WAIT_EVENT.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int wait_event_test(int fd)
{
	int	errs	= 0;

	gsc_label("HSDI4AO4_IOCTL_WAIT_EVENT");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);
	return(errs);
}


