// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/samples/signals/perform.c $
// $Rev: 53419 $
// $Date: 2023-07-20 12:46:41 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _continuous_signals(const args_t* args)
{
	int	errs	= 0;
	s32	fsamp	= 40000;
	s32	ndiv;
	s32	nref;
	s32	nvco;

	//===============================================================
	gsc_label("Analog Input Clock");
	printf("\n");
	gsc_label_level_inc();
	errs	+= hsdi4ao4_fsamp_ai_compute(args->fd, -1, 1, fsamp, 1, &ndiv, &nvco, &nref, NULL);
	errs	+= hsdi4ao4_ai_clock_mode	(args->fd, -1, 1, HSDI4AO4_CLOCK_MODE_OUT, NULL);
	errs	+= hsdi4ao4_ai_enable		(args->fd, -1, 1, HSDI4AO4_AIO_ENABLE_YES, NULL);
	errs	+= hsdi4ao4_gen_a_enable	(args->fd, -1, 1, HSDI4AO4_GEN_ENABLE_YES, NULL);
	errs	+= hsdi4ao4_gen_a_nvco		(args->fd, -1, 1, nvco, NULL);
	errs	+= hsdi4ao4_gen_a_nref		(args->fd, -1, 1, nref, NULL);
	errs	+= hsdi4ao4_gen_a_ndiv		(args->fd, -1, 1, ndiv, NULL);
	errs	+= hsdi4ao4_fsamp_ai_report	(args->fd, -1, 1, -1, NULL);
	gsc_label_level_dec();

	//===============================================================
	gsc_label("Analog Input Trigger");
	printf("\n");
	gsc_label_level_inc();
	errs	+= hsdi4ao4_ai_burst_size	(args->fd, -1, 1, 1, NULL);
	errs	+= hsdi4ao4_ai_burst_enable	(args->fd, -1, 1, HSDI4AO4_BURST_ENABLE_YES, NULL);
	errs	+= hsdi4ao4_gen_b_ndiv		(args->fd, -1, 1, 50, NULL);
	errs	+= hsdi4ao4_gen_b_enable	(args->fd, -1, 1, HSDI4AO4_GEN_ENABLE_YES, NULL);
	gsc_label_level_dec();

	//===============================================================
	gsc_label("Analog Output Clock");
	printf("\n");
	gsc_label_level_inc();
	errs	+= hsdi4ao4_ao_clock_mode	(args->fd, -1, 1, HSDI4AO4_CLOCK_MODE_OUT, NULL);
	errs	+= hsdi4ao4_gen_c_ndiv		(args->fd, -1, 1, 400, NULL);
	errs	+= hsdi4ao4_gen_c_enable	(args->fd, -1, 1, HSDI4AO4_GEN_ENABLE_YES, NULL);
	errs	+= hsdi4ao4_ao_enable		(args->fd, -1, 1, HSDI4AO4_AIO_ENABLE_YES, NULL);
	gsc_label_level_dec();

	//===============================================================
	gsc_label("Waiting");
	os_sleep_ms(args->seconds * 1000);
	printf("Done\n");

	return(errs);
}



//*****************************************************************************
static int _input_software_trigger(const args_t* args)
{
	int		errs	= 0;
	time_t	limit;
	time_t	now;
	int		ret;

	gsc_label("Analog Input SW Trigger");
	printf("\n");
	gsc_label_level_inc();
	errs	+= hsdi4ao4_ai_clock_mode	(args->fd, -1, 1, HSDI4AO4_CLOCK_MODE_OUT, NULL);
	errs	+= hsdi4ao4_gen_b_enable	(args->fd, -1, 1, HSDI4AO4_GEN_ENABLE_NO, NULL);
	errs	+= hsdi4ao4_ai_burst_enable	(args->fd, -1, 1, HSDI4AO4_BURST_ENABLE_YES, NULL);

	// Wait a bit.
	gsc_label("Waiting");
	limit	= time(NULL) + args->seconds;

	for (; errs == 0;)
	{
		ret		= hsdi4ao4_ioctl(args->fd, HSDI4AO4_IOCTL_AI_SW_TRIGGER, NULL);
		errs	+= ret ? 1 : 0;

		now	= time(NULL);

		if (now >= limit)
			break;

		os_sleep_ms(1);
	}

	if (errs == 0)
		printf("Done\n");

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _output_software_clock(const args_t* args)
{
	int		errs	= 0;
	time_t	limit;
	time_t	now;
	int		ret;

	gsc_label("Analog Output SW Clock");
	printf("\n");
	gsc_label_level_inc();
	errs	+= hsdi4ao4_ao_clock_mode	(args->fd, -1, 1, HSDI4AO4_CLOCK_MODE_OUT, NULL);
	errs	+= hsdi4ao4_gen_c_enable	(args->fd, -1, 1, HSDI4AO4_GEN_ENABLE_NO, NULL);

	// Wait a bit.
	gsc_label("Waiting");
	limit	= time(NULL) + args->seconds;

	for (; errs == 0;)
	{
		ret		= hsdi4ao4_ioctl(args->fd, HSDI4AO4_IOCTL_AO_SW_CLOCK, NULL);
		errs	+= ret ? 1 : 0;

		now	= time(NULL);

		if (now >= limit)
			break;

		os_sleep_ms(1);
	}

	if (errs == 0)
		printf("Done\n");

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;

	errs	+= hsdi4ao4_initialize		(args->fd, -1, 1);
	errs	+= hsdi4ao4_trigger_mode	(args->fd, -1, 1, HSDI4AO4_TRIGGER_MODE_OUT, NULL);

	switch (args->sw_flags)
	{
		default:

			errs	= _continuous_signals(args);
			break;

		case SW_FLAG_INPUT_TRIGGER:

			errs	= _input_software_trigger(args);
			break;

		case SW_FLAG_OUTPUT_CLOCK:

			errs	= _output_software_clock(args);
			break;
	}

	errs	+= hsdi4ao4_initialize		(args->fd, -1, 1);
	return(errs);
}



