// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/utils/util_ai_chan_sel.c $
// $Rev: 52566 $
// $Date: 2023-03-08 09:41:05 -0600 (Wed, 08 Mar 2023) $

// 16HSDI4AO4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hsdi4ao4_ai_chan_sel
*
*	Purpose:
*
*		Provide a visual wrapper for the HSDI4AO4_IOCTL_AI_CHAN_SEL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	hsdi4ao4_ai_chan_sel(int fd, int index, int verbose, s32 set, s32* get)
{
	int	errs;
	s32	qty		= 0;
	int	ret;

	if (verbose)
		gsc_label_index("AI Channel Select", index);

	errs	= hsdi4ao4_query(fd, -1, 0, HSDI4AO4_QUERY_CHANNEL_AI_QTY, &qty);

	if (qty == 4)
		set	&= 0xF;
	else if (qty == 2)
		set	&= 0x3;
	else
		set	= 0;

	ret		= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_AI_CHAN_SEL, &set);
	errs	+= ret ? 1 : 0;

	if (verbose)
		printf("%s  (0x%lX)\n", errs ? "FAIL <---" : "PASS", (long) set);

	if (get)
		get[0]	= set;

	return(errs);
}


