// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/utils/util_auto_cal_sts.c $
// $Rev: 53417 $
// $Date: 2023-07-20 12:43:26 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Utilities: source file

#include "main.h"



// This source is retained for backwards compatibility only.

/******************************************************************************
*
*	Function:	hsdi4ao4_auto_cal_sts
*
*	Purpose:
*
*		Provide a visual wrapper for the HSDI4AO4_IOCTL_AUTO_CAL_STS service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	hsdi4ao4_auto_cal_sts(int fd, int index, int verbose, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr		= buf;
	int			ret;
	s32			set;

	if (verbose)
		gsc_label_index("AO Clock Status", index);

	ret		= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_AUTO_CAL_STS, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case HSDI4AO4_AUTO_CAL_STS_ACTIVE:

			ptr	= "Active";
			break;

		case HSDI4AO4_AUTO_CAL_STS_FAIL:

			ptr	= "Failed";
			break;

		case HSDI4AO4_AUTO_CAL_STS_PASS:

			ptr	= "Passed";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


