// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/utils/util_config_ai.c $
// $Rev: 53417 $
// $Date: 2023-07-20 12:43:26 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hsdi4ao4_config_ai
*
*	Purpose:
*
*		Configure the given device's analog input options.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	This is the desired Fsamp rate, or -1 to use the default.
*
*		master	The master clock frequency, or -1.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int hsdi4ao4_config_ai(int fd, int index, int verbose, s32 fsamp, long master_clock)
{
	s32	chans	= 0;
	int	errs	= 0;
	s32	mask;
	s32	ndiv;
	s32	nref;
	s32	nvco;

	if (verbose)
	{
		gsc_label_index("Input Configuration", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= hsdi4ao4_initialize			(fd, index, verbose);

	// Settings must be applied after the initialization call!

	errs	+= hsdi4ao4_rx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,					NULL);
	errs	+= hsdi4ao4_rx_io_overflow		(fd, index, verbose, HSDI4AO4_IO_OVERFLOW_CHECK,		NULL);
	errs	+= hsdi4ao4_rx_io_timeout		(fd, index, verbose, 30,								NULL);
	errs	+= hsdi4ao4_rx_io_underflow		(fd, index, verbose, HSDI4AO4_IO_UNDERFLOW_CHECK,		NULL);

	errs	+= hsdi4ao4_master_clock		(fd, index, verbose, master_clock,						NULL);
	errs	+= hsdi4ao4_data_format			(fd, index, verbose, HSDI4AO4_DATA_FORMAT_OFF_BIN,		NULL);

	errs	+= hsdi4ao4_ai_clock_mode		(fd, index, verbose, HSDI4AO4_CLOCK_MODE_OUT,			NULL);
	errs	+= hsdi4ao4_ai_mode				(fd, index, verbose, HSDI4AO4_AI_MODE_DIFF,				NULL);
	errs	+= hsdi4ao4_ai_buf_thr_lvl		(fd, index, verbose, 200L * 1024,						NULL);
	errs	+= hsdi4ao4_ai_burst_enable		(fd, index, verbose, HSDI4AO4_BURST_ENABLE_NO,			NULL);
	errs	+= hsdi4ao4_ai_range			(fd, index, verbose, HSDI4AO4_RANGE_10V,				NULL);

	errs	+= hsdi4ao4_query				(fd, index, verbose, HSDI4AO4_QUERY_CHANNEL_AI_QTY, &chans);
	mask	= (chans == 4) ? 0xF : ((chans == 2) ? 0x3 : 0x0);
	errs	+= hsdi4ao4_ai_chan_sel			(fd, index, verbose, mask,								NULL);

	errs	+= hsdi4ao4_trigger_mode		(fd, index, verbose, HSDI4AO4_TRIGGER_MODE_OUT,			NULL);
	errs	+= hsdi4ao4_fsamp_ai_compute	(fd, index, verbose, fsamp, 1, &ndiv, &nvco, &nref,		NULL);
	errs	+= hsdi4ao4_gen_a_ndiv			(fd, index, verbose, ndiv,								NULL);
	errs	+= hsdi4ao4_gen_a_nvco			(fd, index, verbose, nvco,								NULL);
	errs	+= hsdi4ao4_gen_a_nref			(fd, index, verbose, nref,								NULL);
	errs	+= hsdi4ao4_gen_a_enable		(fd, index, verbose, HSDI4AO4_GEN_ENABLE_YES,			NULL);
	errs	+= hsdi4ao4_gen_b_enable		(fd, index, verbose, HSDI4AO4_GEN_ENABLE_NO,			NULL);
	errs	+= hsdi4ao4_ai_enable			(fd, index, verbose, HSDI4AO4_AIO_ENABLE_YES,			NULL);

	errs	+= hsdi4ao4_fsamp_ai_report_all	(fd, index, verbose,									NULL);

	// Settings affecting the sample rate or voltage range must generally
	// be applied before autocalibration.
	// Refer to the device user manual for clarification.
	errs	+= hsdi4ao4_autocal				(fd, index, verbose);	// Cable may need disconnecting for this.

	// Interrupt selections should be applied after autocalibration as the
	// driver overwrites the current selection to detect the Autocalibration
	// Done interrupt. If an application intends to detect the Autocalibration
	// Done interrupt, then the interrupt selection may be made beforehand.
	errs	+= hsdi4ao4_irq_enable			(fd, index, verbose, 0,									NULL);

	// The final step is to clear the buffer. This service clears the FIFO and
	// resets the overflow and underflow status bits.
	errs	+= hsdi4ao4_ai_buf_clear		(fd, index, verbose);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


