// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/utils/util_dio_dir_out.c $
// $Rev: 49711 $
// $Date: 2021-10-01 17:06:00 -0500 (Fri, 01 Oct 2021) $

// 16HSDI4AO4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hsdi4ao4_dio_dir_out
*
*	Purpose:
*
*		Provide a visual wrapper for the HSDI4AO4_IOCTL_DIO_DIR_OUT service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	hsdi4ao4_dio_dir_out(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr		= buf;
	int			ret;

	if (verbose)
		gsc_label_index("DIO Outputs", index);

	ret		= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_DIO_DIR_OUT, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case HSDI4AO4_DIO_DIR_OUT_B0_IN_B1_IN:

			ptr	= "Both Bytes In";
			break;

		case HSDI4AO4_DIO_DIR_OUT_B0_OT_B1_IN:

			ptr	= "Byte 0 Out, Byte 1 In";
			break;

		case HSDI4AO4_DIO_DIR_OUT_B0_IN_B1_OT:

			ptr	= "Byte 0 In, Byte 1 Out";
			break;

		case HSDI4AO4_DIO_DIR_OUT_B0_OT_B1_OT:

			ptr	= "Both Bytes Out";
			break;
	}

	if (get)
		get[0]	= set;

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	return(errs);
}


