// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/utils/util_fsamp_ai.c $
// $Rev: 53417 $
// $Date: 2023-07-20 12:43:26 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	LIMIT_MAX(v,max)		(((v) > (max)) ? (max) : (v))
#define	LIMIT_MIN(v,min)		(((v) < (min)) ? (min) : (v))
#define	LIMIT_RANGE(v,min,max)	(LIMIT_MIN((LIMIT_MAX((v),(max))),(min)))



// data types *****************************************************************

typedef struct
{
	double	delta;

	s32		fgen_max;	// Queried value.
	s32		fgen_min;	// Queried value.
	s32		fref;		// device master clock oscillator frequency

	s32		fsamp_want;	// The desired sample rate.
	double	fsamp_got;	// The sample rate achieved.
	s32		fsamp_max;	// Queried value.
	s32		fsamp_min;	// Queried value.

	s32		ndiv;
	s32		ndiv_max;	// Queried value.
	s32		ndiv_min;	// Queried value.

	s32		nref;
	s32		nref_max;	// Queried value. May be reduced to improve noise.
	s32		nref_min;	// Queried value.

	s32		nvco;
	s32		nvco_max;	// Queried value. May be reduced to improve noise.
	s32		nvco_min;	// Queried value.

} data_t;




//*****************************************************************************
static void _fsamp_compute_low(data_t* data)
{
	double	delta;
	s32		fgen;
	double	fsamp;
	s32		ndiv;
	s32		ndiv_max;
	s32		ndiv_min;
	s32		nref;
	s32		nref_max;
	s32		nref_min;
	s32		nvco;

	for (nvco = data->nvco_min; nvco <= data->nvco_max; nvco++)
	{
		// Compute the usable Nref range.
		nref_max	= (s32) ((float) data->fref * nvco / data->fgen_min / 2);
		nref_min	= (s32) ((float) data->fref * nvco / data->fgen_max / 2);

		if ((nref_max < data->nref_min) || (nref_max > data->nref_max))
			continue;

		if ((nref_min < data->nref_min) || (nref_min > data->nref_max))
			continue;

		for (nref = nref_min; nref <= nref_max; nref++)
		{
			// Compute the resulting Fgen frequency.
			fgen	= (s32) ((long) data->fref * nvco / nref / 2);

			if ((fgen < data->fgen_min) || (fgen > data->fgen_max))
				continue;

			// Compute the usable Ndiv range.
			ndiv	= (long) fgen / data->fsamp_want / 32;

			if ((ndiv < data->ndiv_min) || (ndiv > data->ndiv_max))
				continue;

			ndiv_max	= ndiv + 1;
			ndiv_min	= ndiv - 1;

			ndiv_max	= LIMIT_RANGE(ndiv_max, data->ndiv_min, data->ndiv_max);
			ndiv_min	= LIMIT_RANGE(ndiv_min, data->ndiv_min, data->ndiv_max);

			for (ndiv = ndiv_min; ndiv <= ndiv_max; ndiv++)
			{
				// COmpute the resulting sample rate.
				fsamp	= (double) fgen / ndiv / 32;
				delta	= fsamp - data->fsamp_want;

				if (delta < 0)
					delta	= -delta;

				if (delta < data->delta)
				{
					data->delta		= delta;
					data->fsamp_got	= fsamp;
					data->ndiv		= ndiv;
					data->nref		= nref;
					data->nvco		= nvco;
				}
			}
		}
	}
}



//*****************************************************************************
static void _fsamp_compute_high(data_t* data)
{
	double	delta;
	s32		fgen;
	double	fsamp;
	s32		ndiv;
	s32		ndiv_max;
	s32		ndiv_min;
	s32		nref;
	s32		nref_max;
	s32		nref_min;
	s32		nvco;

	for (nvco = data->nvco_min; nvco <= data->nvco_max; nvco++)
	{
		// Compute the usable Nref range.
		nref_max	= (s32) ((float) data->fref * nvco / data->fgen_min / 2);
		nref_min	= (s32) ((float) data->fref * nvco / data->fgen_max / 2);

		if ((nref_max < data->nref_min) || (nref_max > data->nref_max))
			continue;

		if ((nref_min < data->nref_min) || (nref_min > data->nref_max))
			continue;

		for (nref = nref_min; nref <= nref_max; nref++)
		{
			// Compute the resulting Fgen frequency.
			fgen	= (s32) ((long) data->fref * nvco / nref / 2);

			if ((fgen < data->fgen_min) || (fgen > data->fgen_max))
				continue;

			// Compute the usable Ndiv range.
			ndiv	= (long) fgen / data->fsamp_want / 32;

			if ((ndiv < data->ndiv_min) || (ndiv > data->ndiv_max))
				continue;

			ndiv_max	= ndiv + 1;
			ndiv_min	= ndiv - 1;

			ndiv_max	= LIMIT_RANGE(ndiv_max, data->ndiv_min, data->ndiv_max);
			ndiv_min	= LIMIT_RANGE(ndiv_min, data->ndiv_min, data->ndiv_max);

			// COmpute the resulting sample rate.
			fsamp	= (double) fgen / 16;
			delta	= fsamp - data->fsamp_want;

			if (delta < 0)
				delta	= -delta;

			if (delta < data->delta)
			{
				data->delta		= delta;
				data->fsamp_got	= fsamp;
				data->ndiv		= ndiv;
				data->nref		= nref;
				data->nvco		= nvco;
			}
		}
	}
}



//*****************************************************************************
int hsdi4ao4_fsamp_ai_compute(
	int		fd,
	int		index,
	int		verbose,
	s32		fsamp,
	s32		low_noise,
	s32*	ndiv,
	s32*	nvco,
	s32*	nref,
	double*	rate)
{
	data_t	data;
	int		errs	= 0;

	if (verbose)
	{
		gsc_label_index("AI Sample Rate", index);
		gsc_label_long_comma(fsamp);
		printf(" S/S");
	}

	memset(&data, 0, sizeof(data));

	errs	+= hsdi4ao4_query(fd, -1, 0, HSDI4AO4_QUERY_MASTER_CLOCK_DEF, &data.fref);

	errs	+= hsdi4ao4_query(fd, -1, 0, HSDI4AO4_QUERY_FSAMP_MAX, &data.fsamp_max);
	errs	+= hsdi4ao4_query(fd, -1, 0, HSDI4AO4_QUERY_FSAMP_MIN, &data.fsamp_min);

	errs	+= hsdi4ao4_query(fd, -1, 0, HSDI4AO4_QUERY_GEN_A_MAX, &data.fgen_max);
	errs	+= hsdi4ao4_query(fd, -1, 0, HSDI4AO4_QUERY_GEN_A_MIN, &data.fgen_min);

	errs	+= hsdi4ao4_query(fd, -1, 0, HSDI4AO4_QUERY_GEN_A_NDIV_MAX, &data.ndiv_max);
	errs	+= hsdi4ao4_query(fd, -1, 0, HSDI4AO4_QUERY_GEN_A_NDIV_MIN, &data.ndiv_min);

	errs	+= hsdi4ao4_query(fd, -1, 0, HSDI4AO4_QUERY_GEN_A_NREF_MAX, &data.nref_max);
	errs	+= hsdi4ao4_query(fd, -1, 0, HSDI4AO4_QUERY_GEN_A_NREF_MIN, &data.nref_min);

	errs	+= hsdi4ao4_query(fd, -1, 0, HSDI4AO4_QUERY_GEN_A_NVCO_MAX, &data.nvco_max);
	errs	+= hsdi4ao4_query(fd, -1, 0, HSDI4AO4_QUERY_GEN_A_NVCO_MIN, &data.nvco_min);

	data.delta		= 100000000.0;
	data.fsamp_want	= LIMIT_RANGE(fsamp, data.fsamp_min, data.fsamp_max);

	if (low_noise)
	{
		data.nref_max	= 250;
		data.nvco_max	= 250;
	}

	if (fsamp > 600000)
	{
		_fsamp_compute_high(&data);
	}
	else	// <= 600,000
	{
		data.ndiv_min	= 1;
		_fsamp_compute_low(&data);
	}

	if (ndiv)
		ndiv[0]	= data.ndiv;

	if (nvco)
		nvco[0]	= data.nvco;

	if (nref)
		nref[0]	= data.nref;

	if (rate)
		rate[0]	= (s32) (data.fsamp_got + 0.5);

	if (verbose)
	{
		printf("%s  (", errs ? " FAIL <---" : "");
		printf("Fref ");
		gsc_label_long_comma((long) data.fref);
		printf(", Ndiv ");
		gsc_label_long_comma((long) data.ndiv);
		printf(", Nvco ");
		gsc_label_long_comma((long) data.nvco);
		printf(", Nref ");
		gsc_label_long_comma((long) data.nref);
		printf(", Fsamp ");
		gsc_label_float_comma(data.fsamp_got, 0, 3);
		printf(")\n");
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	hsdi4ao4_fsamp_ai_get
*
*	Purpose:
*
*		Determine the input sample rate.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		verbose	Work verbosely?
*
*		get		Store the sample rate here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int hsdi4ao4_fsamp_ai_get(int fd, int verbose, s32* get)
{
	s32	enable	= -1;
	int	errs	= 0;
	s32	mode	= -1;
	s32	ndiv	= -1;
	s32	nvco	= -1;
	s32	nref	= -1;
	s32	master;
	s32	set		= 0;

	if (verbose)
		gsc_label("AI Sample Rate");

	errs	+= hsdi4ao4_ai_clock_mode(fd, -1, 0, -1, &mode);
	errs	+= hsdi4ao4_master_clock(fd, -1, 0, -1, &master);
	errs	+= hsdi4ao4_gen_a_enable(fd, -1, 0, -1, &enable);
	errs	+= hsdi4ao4_gen_a_ndiv(fd, -1, 0, -1, &ndiv);
	errs	+= hsdi4ao4_gen_a_nvco(fd, -1, 0, -1, &nvco);
	errs	+= hsdi4ao4_gen_a_nref(fd, -1, 0, -1, &nref);

	if (errs)
	{
		set	= 0;

		if (verbose)
			printf("FAIL <---  (%d. service errors)\n", __LINE__);
	}
	else if (mode == HSDI4AO4_CLOCK_MODE_IN)
	{
		set	= 0;

		if (verbose)
			printf("Unknown  (Clock Mode = Input)\n");
	}
	else if (enable == HSDI4AO4_GEN_ENABLE_NO)
	{
		set	= 0;

		if (verbose)
			printf("0 S/S  (Rate Gen. A is disabled.)\n");
	}
	else
	{
		if (ndiv == 0)
		{
			set = (master / 2 * nvco) / (16 * nref);  // = ((master/2) * (nvco/nref)) / 16
		}
		else
		{
			set = (master / 2 * nvco) / (32 * ndiv * nref);  // = ((master/2) * (nvco/nref)) / (32*ndiv)
		}

		if (verbose)
		{
			gsc_label_long_comma(set);
			printf(" S/S  (Source = Rate Gen A, Ref = ");
			gsc_label_long_comma(master);
			printf(", Nvco = ");
			gsc_label_long_comma(nvco);
			printf(", Nref = ");
			gsc_label_long_comma(nref);
			printf(", Ndiv = ");
			gsc_label_long_comma(ndiv);
			printf(")\n");
		}
	}

	if (get)
		get[0]	= set;

	return(errs);
}



/******************************************************************************
*
*	Function:	hsdi4ao4_fsamp_ai_report
*
*	Purpose:
*
*		Determine and report the input sample rate for the specified channel.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		verbose	Work verbosely?
*
*		index	The index of the device to access. Ignore if < 0.
*
*		chan	The index of the channel of interest. Ignore if < 0.
*
*		get		Store the sample rate here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int hsdi4ao4_fsamp_ai_report(int fd, int index, int verbose, int chan, s32* get)
{
	s32		bit		= 0x1L << chan;
	char	buf[64];
	int		errs;
	s32		select;
	s32		set		= 0;

	if (verbose)
	{
		if (chan < 0)
			sprintf(buf, "AI Channels");
		else
			sprintf(buf, "AI Channel %d", chan);

		gsc_label_index(buf, index);
	}

	errs	= hsdi4ao4_ai_chan_sel(fd, -1, 0, -1, &select);

	if (errs)
	{
		if (verbose)
			printf("FAIL <---  (channel select query)\n");
	}
	else if ((chan >= 0) && ((select & bit) == 0))
	{
		// The channel is disabled.

		if (verbose)
			printf("0 S/S  (Channel Disabled)\n");
	}
	else
	{
		// The channel is enabled.
		errs	+= hsdi4ao4_fsamp_ai_get(fd, 0, &set);

		if (verbose)
		{
			gsc_label_long_comma(set);
			printf(" S/S\n");
		}
	}

	if (get)
		get[0]	= set;

	return(errs);
}



/******************************************************************************
*
*	Function:	hsdi4ao4_fsamp_ai_report_all
*
*	Purpose:
*
*		Determine and report the input sample rate for all channels.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	Report overall sample rate here.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int hsdi4ao4_fsamp_ai_report_all(int fd, int index, int verbose, s32* fsamp)
{
	s32	chans;
	int	chan;
	int	errs;
	s32	sps;
	s32	total	= 0;

	if (verbose)
		gsc_label_index("AI Sample Rates", index);

	errs	= hsdi4ao4_query(fd, -1, 0, HSDI4AO4_QUERY_CHANNEL_AI_QTY, &chans);

	if (errs)
	{
		if (verbose)
			printf("FAIL <---  (channel count query)\n");
	}
	else
	{
		if (verbose)
		{
			printf("\n");
			gsc_label_level_inc();
		}

		for (chan = 0; chan < chans; chan++)
		{
			sps		= 0;
			errs	+= hsdi4ao4_fsamp_ai_report(fd, index, verbose, chan, &sps);
			total	+= sps;
		}

		if (verbose)
		{
			gsc_label("Overall AI Rate");
			gsc_label_long_comma(total);
			printf(" S/S\n");
			gsc_label_level_dec();
		}
	}

	if (fsamp)
		fsamp[0]	= total;

	return(errs);
}


