// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/utils/util_query.c $
// $Rev: 53417 $
// $Date: 2023-07-20 12:43:26 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hsdi4ao4_query
*
*	Purpose:
*
*		Provide a visual wrapper for the HSDI4AO4_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	hsdi4ao4_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case HSDI4AO4_QUERY_AUTOCAL_MS:			ptr	= "Autocal Duration";		break;
		case HSDI4AO4_QUERY_CHANNEL_AI_MAX:		ptr	= "Max AI Channels";		break;
		case HSDI4AO4_QUERY_CHANNEL_AI_QTY:		ptr	= "Installed AI Channels";	break;
		case HSDI4AO4_QUERY_CHANNEL_AO_MAX:		ptr	= "Max AO Channels";		break;
		case HSDI4AO4_QUERY_CHANNEL_AO_QTY:		ptr	= "Installed AO Channels";	break;
		case HSDI4AO4_QUERY_COUNT:				ptr	= "Query Options";			break;
		case HSDI4AO4_QUERY_DEVICE_TYPE:		ptr	= "Device Type";			break;
		case HSDI4AO4_QUERY_FIFO_SIZE_RX:		ptr	= "Rx FIFO Size";			break;
		case HSDI4AO4_QUERY_FIFO_SIZE_TX:		ptr	= "Tx FIFO Size";			break;
		case HSDI4AO4_QUERY_FSAMP_MAX:			ptr	= "Fsamp Max";				break;
		case HSDI4AO4_QUERY_FSAMP_MIN:			ptr	= "Fsamp Min";				break;
		case HSDI4AO4_QUERY_GEN_A_MAX:			ptr	= "Gen. A Max";				break;
		case HSDI4AO4_QUERY_GEN_A_MIN:			ptr	= "Gen. A Max";				break;
		case HSDI4AO4_QUERY_GEN_A_NDIV_MAX:		ptr	= "Gen. A Ndiv Max";		break;
		case HSDI4AO4_QUERY_GEN_A_NDIV_MIN:		ptr	= "Gen. A Ndiv Min";		break;
		case HSDI4AO4_QUERY_GEN_A_NREF_MAX:		ptr	= "Gen. A Nref Max";		break;
		case HSDI4AO4_QUERY_GEN_A_NREF_MIN:		ptr	= "Gen. A Nref Min";		break;
		case HSDI4AO4_QUERY_GEN_A_NVCO_MAX:		ptr	= "Gen. A Nvco Max";		break;
		case HSDI4AO4_QUERY_GEN_A_NVCO_MIN:		ptr	= "Gen. A Nvco Min";		break;
		case HSDI4AO4_QUERY_GEN_B_NDIV_MAX:		ptr	= "Gen. B Ndiv Max";		break;
		case HSDI4AO4_QUERY_GEN_B_NDIV_MIN:		ptr	= "Gen. B Ndiv Min";		break;
		case HSDI4AO4_QUERY_GEN_C_NDIV_MAX:		ptr	= "Gen. C Ndiv Max";		break;
		case HSDI4AO4_QUERY_GEN_C_NDIV_MIN:		ptr	= "Gen. C Ndiv Min";		break;
		case HSDI4AO4_QUERY_INIT_MS:			ptr	= "Init. Duration";			break;
		case HSDI4AO4_QUERY_MASTER_CLOCK_DEF:	ptr	= "Master Clock";			break;
		case HSDI4AO4_QUERY_MCAR:				ptr	= "Reg: MCAR";				break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= hsdi4ao4_ioctl(fd, HSDI4AO4_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			gsc_label_long_comma_buf(set, buf);
			break;

		case HSDI4AO4_QUERY_CHANNEL_AI_MAX:
		case HSDI4AO4_QUERY_CHANNEL_AI_QTY:
		case HSDI4AO4_QUERY_CHANNEL_AO_MAX:
		case HSDI4AO4_QUERY_CHANNEL_AO_QTY:
		case HSDI4AO4_QUERY_COUNT:
		case HSDI4AO4_QUERY_GEN_A_NDIV_MAX:
		case HSDI4AO4_QUERY_GEN_A_NDIV_MIN:
		case HSDI4AO4_QUERY_GEN_A_NREF_MAX:
		case HSDI4AO4_QUERY_GEN_A_NREF_MIN:
		case HSDI4AO4_QUERY_GEN_A_NVCO_MAX:
		case HSDI4AO4_QUERY_GEN_A_NVCO_MIN:
		case HSDI4AO4_QUERY_GEN_B_NDIV_MAX:
		case HSDI4AO4_QUERY_GEN_B_NDIV_MIN:
		case HSDI4AO4_QUERY_GEN_C_NDIV_MAX:
		case HSDI4AO4_QUERY_GEN_C_NDIV_MIN:

			gsc_label_long_comma_buf(set, buf);
			break;

		case HSDI4AO4_QUERY_GEN_A_MAX:
		case HSDI4AO4_QUERY_GEN_A_MIN:
		case HSDI4AO4_QUERY_MASTER_CLOCK_DEF:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " Hz");
			break;

		case HSDI4AO4_QUERY_AUTOCAL_MS:
		case HSDI4AO4_QUERY_INIT_MS:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " ms");
			break;

		case HSDI4AO4_QUERY_FSAMP_MAX:
		case HSDI4AO4_QUERY_FSAMP_MIN:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " S/S");
			break;

		case HSDI4AO4_QUERY_MCAR:

			sprintf(buf, "%s", set ? "Supported" : "Not Supported");
			break;

		case HSDI4AO4_QUERY_FIFO_SIZE_RX:
		case HSDI4AO4_QUERY_FIFO_SIZE_TX:

			ptr	= "";
			tmp	= set;

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "K";
				tmp	/= 1024;
			}

			gsc_label_long_comma_buf(tmp, buf);
			strcat(buf, ptr);
			strcat(buf, " Words");
			break;

		case HSDI4AO4_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_16HSDI4AO4)
			{
				strcpy(buf, "16HSDI4AO4");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


