// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16HSDI4AO4/utils/write.c $
// $Rev: 53417 $
// $Date: 2023-07-20 12:43:26 -0500 (Thu, 20 Jul 2023) $

// 16HSDI4AO4: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hsdi4ao4_write_util
*
*	Purpose:
*
*		Implement a visual wrapper around the device write call.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		src		The source for the data to write.
*
*		bytes	The voume of data to write.
*
*		sent	If non-NULL, then this is the number of bytes written.
*
*	Returned:
*
*		>= 0	The volume of data written.
*		< 0		The negative of the number of errors seen.
*
******************************************************************************/

int hsdi4ao4_write_util(int fd, int index, int verbose, const void* src, size_t bytes, size_t* sent)
{
	int	errs;
	int	ret;

	if (verbose)
		gsc_label_index("Writing", index);

	ret	= hsdi4ao4_write(fd, src, bytes);
	errs	= (ret < 0) ? 1 : 0;

	if (verbose == 0)
	{
	}
	else if (ret >= 0)
	{
		printf("PASS  (%d Byte%s)\n", ret, (ret == 1) ? "" : "s");
	}
	else
	{
		printf(	"FAIL <---  (requested %ld Byte%s, ret = %ld)\n",
				(long) bytes,
				(bytes == 1) ? "" : "s",
				(long) ret);
	}

	if (sent)
		sent[0]	= (ret >= 0) ? ret : 0;

	return(errs);
}


